"use strict";
/**
 * @license
 * Copyright 2018 Google Inc. All Rights Reserved.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * =============================================================================
 */
Object.defineProperty(exports, "__esModule", { value: true });
var tf = require("../index");
var test_util_1 = require("../test_util");
var jasmine_util_1 = require("../jasmine_util");
jasmine_util_1.describeWithFlags('equal', test_util_1.ALL_ENVS, function () {
    it('Tensor1D - int32', function () {
        var a = tf.tensor1d([1, 4, 5], 'int32');
        var b = tf.tensor1d([2, 3, 5], 'int32');
        test_util_1.expectArraysClose(tf.equal(a, b), [0, 0, 1]);
        a = tf.tensor1d([2, 2, 2], 'int32');
        b = tf.tensor1d([2, 2, 2], 'int32');
        test_util_1.expectArraysClose(tf.equal(a, b), [1, 1, 1]);
        a = tf.tensor1d([0, 0], 'int32');
        b = tf.tensor1d([3, 3], 'int32');
        test_util_1.expectArraysClose(tf.equal(a, b), [0, 0]);
    });
    it('Tensor1D - float32', function () {
        var a = tf.tensor1d([1.1, 4.1, 5.1], 'float32');
        var b = tf.tensor1d([2.2, 3.2, 5.1], 'float32');
        test_util_1.expectArraysClose(tf.equal(a, b), [0, 0, 1]);
        a = tf.tensor1d([2.31, 2.31, 2.31], 'float32');
        b = tf.tensor1d([2.31, 2.31, 2.31], 'float32');
        test_util_1.expectArraysClose(tf.equal(a, b), [1, 1, 1]);
        a = tf.tensor1d([0.45, 0.123], 'float32');
        b = tf.tensor1d([3.123, 3.321], 'float32');
        test_util_1.expectArraysClose(tf.equal(a, b), [0, 0]);
    });
    it('upcasts when dtypes dont match', function () {
        var a = [1.1, 4.1, 5];
        var b = [2.2, 3.2, 5];
        var res = tf.equal(tf.tensor(a, [3], 'float32'), tf.tensor(b, [3], 'int32'));
        expect(res.dtype).toBe('bool');
        expect(res.shape).toEqual([3]);
        test_util_1.expectArraysClose(res, [0, 0, 1]);
        res =
            tf.equal(tf.tensor(a, [3], 'int32'), tf.tensor(b, [3], 'bool'));
        expect(res.dtype).toBe('bool');
        expect(res.shape).toEqual([3]);
        test_util_1.expectArraysClose(res, [1, 0, 0]);
    });
    it('TensorLike', function () {
        var a = [1.1, 4.1, 5.1];
        var b = [2.2, 3.2, 5.1];
        test_util_1.expectArraysClose(tf.equal(a, b), [0, 0, 1]);
    });
    it('TensorLike chained', function () {
        var a = tf.tensor1d([1.1, 4.1, 5.1], 'float32');
        var b = [2.2, 3.2, 5.1];
        test_util_1.expectArraysClose(a.equal(b), [0, 0, 1]);
    });
    it('mismatched Tensor1D shapes - int32', function () {
        var a = tf.tensor1d([1, 2], 'int32');
        var b = tf.tensor1d([1, 2, 3], 'int32');
        var f = function () {
            tf.equal(a, b);
        };
        expect(f).toThrowError();
    });
    it('mismatched Tensor1D shapes - float32', function () {
        var a = tf.tensor1d([1.1, 2.1], 'float32');
        var b = tf.tensor1d([1.1, 2.1, 3.1], 'float32');
        var f = function () {
            tf.equal(a, b);
        };
        expect(f).toThrowError();
    });
    it('NaNs in Tensor1D - float32', function () {
        var a = tf.tensor1d([1.1, NaN, 2.1], 'float32');
        var b = tf.tensor1d([2.1, 3.1, NaN], 'float32');
        test_util_1.expectArraysClose(tf.equal(a, b), [0, 0, 0]);
    });
    it('scalar and 1D broadcast', function () {
        var a = tf.scalar(2);
        var b = tf.tensor1d([1, 2, 3, 4, 5, 2]);
        var res = tf.equal(a, b);
        expect(res.dtype).toBe('bool');
        expect(res.shape).toEqual([6]);
        test_util_1.expectArraysEqual(res, [0, 1, 0, 0, 0, 1]);
    });
    // Tensor2D:
    it('Tensor2D - int32', function () {
        var a = tf.tensor2d([[1, 4, 5], [8, 9, 12]], [2, 3], 'int32');
        var b = tf.tensor2d([[2, 3, 6], [7, 10, 11]], [2, 3], 'int32');
        test_util_1.expectArraysClose(tf.equal(a, b), [0, 0, 0, 0, 0, 0]);
        a = tf.tensor2d([[0, 0], [1, 1]], [2, 2], 'int32');
        b = tf.tensor2d([[0, 0], [1, 1]], [2, 2], 'int32');
        test_util_1.expectArraysClose(tf.equal(a, b), [1, 1, 1, 1]);
    });
    it('Tensor2D - float32', function () {
        var a = tf.tensor2d([[1.1, 4.1, 5.1], [8.1, 9.1, 12.1]], [2, 3], 'float32');
        var b = tf.tensor2d([[2.1, 4.1, 5.1], [7.1, 10.1, 11.1]], [2, 3], 'float32');
        test_util_1.expectArraysClose(tf.equal(a, b), [0, 1, 1, 0, 0, 0]);
        a = tf.tensor2d([[0.2, 0.2], [1.2, 1.2]], [2, 2], 'float32');
        b = tf.tensor2d([[0.2, 0.2], [1.2, 1.2]], [2, 2], 'float32');
        test_util_1.expectArraysClose(tf.equal(a, b), [1, 1, 1, 1]);
    });
    it('broadcasting Tensor2D shapes - int32', function () {
        var a = tf.tensor2d([[3], [7]], [2, 1], 'int32');
        var b = tf.tensor2d([[2, 3, 4], [7, 8, 9]], [2, 3], 'int32');
        test_util_1.expectArraysClose(tf.equal(a, b), [0, 1, 0, 1, 0, 0]);
    });
    it('broadcasting Tensor2D shapes - float32', function () {
        var a = tf.tensor2d([[1.1], [7.1]], [2, 1], 'float32');
        var b = tf.tensor2d([[0.1, 1.1, 2.1], [7.1, 8.1, 9.1]], [2, 3], 'float32');
        test_util_1.expectArraysClose(tf.equal(a, b), [0, 1, 0, 1, 0, 0]);
    });
    it('NaNs in Tensor2D - float32', function () {
        var a = tf.tensor2d([[1.1, NaN], [1.1, NaN]], [2, 2], 'float32');
        var b = tf.tensor2d([[0.1, NaN], [1.1, NaN]], [2, 2], 'float32');
        test_util_1.expectArraysClose(tf.equal(a, b), [0, 0, 1, 0]);
    });
    it('2D and 2D broadcast each with 1 dim', function () {
        var a = tf.tensor2d([1, 2, 5], [1, 3]);
        var b = tf.tensor2d([5, 1], [2, 1]);
        var res = tf.equal(a, b);
        expect(res.dtype).toBe('bool');
        expect(res.shape).toEqual([2, 3]);
        test_util_1.expectArraysEqual(res, [0, 0, 1, 1, 0, 0]);
    });
    it('2D and scalar broadcast', function () {
        var a = tf.tensor2d([1, 2, 3, 2, 5, 6], [2, 3]);
        var b = tf.scalar(2);
        var res = tf.equal(a, b);
        expect(res.dtype).toBe('bool');
        expect(res.shape).toEqual([2, 3]);
        test_util_1.expectArraysEqual(res, [0, 1, 0, 1, 0, 0]);
    });
    // Tensor3D:
    it('Tensor3D - int32', function () {
        var a = tf.tensor3d([[[1], [4], [5]], [[8], [9], [12]]], [2, 3, 1], 'int32');
        var b = tf.tensor3d([[[2], [3], [6]], [[7], [10], [12]]], [2, 3, 1], 'int32');
        test_util_1.expectArraysClose(tf.equal(a, b), [0, 0, 0, 0, 0, 1]);
        a = tf.tensor3d([[[0], [0], [0]], [[1], [1], [1]]], [2, 3, 1], 'int32');
        b = tf.tensor3d([[[0], [0], [0]], [[1], [1], [1]]], [2, 3, 1], 'int32');
        test_util_1.expectArraysClose(tf.equal(a, b), [1, 1, 1, 1, 1, 1]);
    });
    it('Tensor3D - float32', function () {
        var a = tf.tensor3d([[[1.1], [4.1], [5.1]], [[8.1], [9.1], [12.1]]], [2, 3, 1], 'float32');
        var b = tf.tensor3d([[[2.1], [3.1], [6.1]], [[7.1], [10.1], [12.1]]], [2, 3, 1], 'float32');
        test_util_1.expectArraysClose(tf.equal(a, b), [0, 0, 0, 0, 0, 1]);
        a = tf.tensor3d([[[0.1], [0.1], [0.1]], [[1.1], [1.1], [1.1]]], [2, 3, 1], 'float32');
        b = tf.tensor3d([[[0.1], [0.1], [0.1]], [[1.1], [1.1], [1.1]]], [2, 3, 1], 'float32');
        test_util_1.expectArraysClose(tf.equal(a, b), [1, 1, 1, 1, 1, 1]);
    });
    it('broadcasting Tensor3D shapes - int32', function () {
        var a = tf.tensor3d([[[1, 0], [2, 3], [4, 5]], [[6, 7], [9, 8], [10, 11]]], [2, 3, 2], 'int32');
        var b = tf.tensor3d([[[1], [2], [3]], [[7], [10], [9]]], [2, 3, 1], 'int32');
        test_util_1.expectArraysClose(tf.equal(a, b), [1, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0]);
    });
    it('broadcasting Tensor3D shapes - float32', function () {
        var a = tf.tensor3d([
            [[1.1, 0.1], [2.1, 3.1], [4.1, 5.1]],
            [[6.1, 7.1], [9.1, 8.1], [10.1, 11.1]]
        ], [2, 3, 2], 'float32');
        var b = tf.tensor3d([[[1.1], [2.1], [3.1]], [[7.1], [10.1], [9.1]]], [2, 3, 1], 'float32');
        test_util_1.expectArraysClose(tf.equal(a, b), [1, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0]);
    });
    it('NaNs in Tensor3D - float32', function () {
        var a = tf.tensor3d([[[1.1], [NaN], [1.1]], [[0.1], [0.1], [0.1]]], [2, 3, 1], 'float32');
        var b = tf.tensor3d([[[0.1], [0.1], [1.1]], [[1.1], [0.1], [NaN]]], [2, 3, 1], 'float32');
        test_util_1.expectArraysClose(tf.equal(a, b), [0, 0, 1, 0, 1, 0]);
    });
    it('3D and scalar', function () {
        var a = tf.tensor3d([1, 2, 3, 4, 5, -1], [2, 3, 1]);
        var b = tf.scalar(-1);
        var res = tf.equal(a, b);
        expect(res.dtype).toBe('bool');
        expect(res.shape).toEqual([2, 3, 1]);
        test_util_1.expectArraysEqual(res, [0, 0, 0, 0, 0, 1]);
    });
    // Tensor4D:
    it('Tensor4D - int32', function () {
        var a = tf.tensor4d([1, 4, 5, 8], [2, 2, 1, 1], 'int32');
        var b = tf.tensor4d([2, 3, 6, 8], [2, 2, 1, 1], 'int32');
        test_util_1.expectArraysClose(tf.equal(a, b), [0, 0, 0, 1]);
        a = tf.tensor4d([0, 1, 2, 3], [2, 2, 1, 1], 'int32');
        b = tf.tensor4d([0, 1, 2, 3], [2, 2, 1, 1], 'int32');
        test_util_1.expectArraysClose(tf.equal(a, b), [1, 1, 1, 1]);
        a = tf.tensor4d([1, 1, 1, 1], [2, 2, 1, 1], 'int32');
        b = tf.tensor4d([2, 2, 2, 2], [2, 2, 1, 1], 'int32');
        test_util_1.expectArraysClose(tf.equal(a, b), [0, 0, 0, 0]);
    });
    it('Tensor4D - float32', function () {
        var a = tf.tensor4d([1.1, 4.1, 5.1, 8.1], [2, 2, 1, 1], 'float32');
        var b = tf.tensor4d([2.1, 3.1, 6.1, 8.1], [2, 2, 1, 1], 'float32');
        test_util_1.expectArraysClose(tf.equal(a, b), [0, 0, 0, 1]);
        a = tf.tensor4d([0.1, 1.1, 2.2, 3.3], [2, 2, 1, 1], 'float32');
        b = tf.tensor4d([0.1, 1.1, 2.2, 3.3], [2, 2, 1, 1], 'float32');
        test_util_1.expectArraysClose(tf.equal(a, b), [1, 1, 1, 1]);
        a = tf.tensor4d([0.1, 0.1, 0.1, 0.1], [2, 2, 1, 1], 'float32');
        b = tf.tensor4d([1.1, 1.1, 1.1, 1.1], [2, 2, 1, 1], 'float32');
        test_util_1.expectArraysClose(tf.equal(a, b), [0, 0, 0, 0]);
    });
    it('broadcasting Tensor4D shapes - int32', function () {
        var a = tf.tensor4d([1, 2, 5, 9], [2, 2, 1, 1], 'int32');
        var b = tf.tensor4d([[[[1, 2]], [[3, 4]]], [[[5, 6]], [[7, 8]]]], [2, 2, 1, 2], 'int32');
        test_util_1.expectArraysClose(tf.equal(a, b), [1, 0, 0, 0, 1, 0, 0, 0]);
    });
    it('broadcasting Tensor4D shapes - float32', function () {
        var a = tf.tensor4d([1.1, 2.1, 5.1, 9.1], [2, 2, 1, 1], 'float32');
        var b = tf.tensor4d([[[[1.1, 2.1]], [[3.1, 4.1]]], [[[5.1, 6.1]], [[7.1, 8.1]]]], [2, 2, 1, 2], 'float32');
        test_util_1.expectArraysClose(tf.equal(a, b), [1, 0, 0, 0, 1, 0, 0, 0]);
    });
    it('NaNs in Tensor4D - float32', function () {
        var a = tf.tensor4d([1.1, NaN, 1.1, 0.1], [2, 2, 1, 1], 'float32');
        var b = tf.tensor4d([0.1, 1.1, 1.1, NaN], [2, 2, 1, 1], 'float32');
        test_util_1.expectArraysClose(tf.equal(a, b), [0, 0, 1, 0]);
    });
    it('throws when passed a as a non-tensor', function () {
        expect(function () { return tf.equal({}, tf.scalar(1)); })
            .toThrowError(/Argument 'a' passed to 'equal' must be a Tensor/);
    });
    it('throws when passed b as a non-tensor', function () {
        expect(function () { return tf.equal(tf.scalar(1), {}); })
            .toThrowError(/Argument 'b' passed to 'equal' must be a Tensor/);
    });
    it('accepts a tensor-like object', function () {
        var a = [1, 4, 5];
        var b = [2, 3, 5];
        test_util_1.expectArraysClose(tf.equal(a, b), [0, 0, 1]);
    });
});
jasmine_util_1.describeWithFlags('equalStrict', test_util_1.ALL_ENVS, function () {
    it('Tensor1D - int32', function () {
        var a = tf.tensor1d([1, 4, 5], 'int32');
        var b = tf.tensor1d([2, 3, 5], 'int32');
        test_util_1.expectArraysClose(tf.equalStrict(a, b), [0, 0, 1]);
        a = tf.tensor1d([2, 2, 2], 'int32');
        b = tf.tensor1d([2, 2, 2], 'int32');
        test_util_1.expectArraysClose(tf.equalStrict(a, b), [1, 1, 1]);
        a = tf.tensor1d([0, 0], 'int32');
        b = tf.tensor1d([3, 3], 'int32');
        test_util_1.expectArraysClose(tf.equalStrict(a, b), [0, 0]);
    });
    it('Tensor1D - float32', function () {
        var a = tf.tensor1d([1.1, 4.1, 5.1], 'float32');
        var b = tf.tensor1d([2.2, 3.2, 5.1], 'float32');
        test_util_1.expectArraysClose(tf.equalStrict(a, b), [0, 0, 1]);
        a = tf.tensor1d([2.31, 2.31, 2.31], 'float32');
        b = tf.tensor1d([2.31, 2.31, 2.31], 'float32');
        test_util_1.expectArraysClose(tf.equalStrict(a, b), [1, 1, 1]);
        a = tf.tensor1d([0.45, 0.123], 'float32');
        b = tf.tensor1d([3.123, 3.321], 'float32');
        test_util_1.expectArraysClose(tf.equalStrict(a, b), [0, 0]);
    });
    it('mismatched Tensor1D shapes - int32', function () {
        var a = tf.tensor1d([1, 2], 'int32');
        var b = tf.tensor1d([1, 2, 3], 'int32');
        var f = function () {
            tf.equalStrict(a, b);
        };
        expect(f).toThrowError();
    });
    it('mismatched Tensor1D shapes - float32', function () {
        var a = tf.tensor1d([1.1, 2.1], 'float32');
        var b = tf.tensor1d([1.1, 2.1, 3.1], 'float32');
        var f = function () {
            tf.equalStrict(a, b);
        };
        expect(f).toThrowError();
    });
    it('NaNs in Tensor1D - float32', function () {
        var a = tf.tensor1d([1.1, NaN, 2.1], 'float32');
        var b = tf.tensor1d([2.1, 3.1, NaN], 'float32');
        test_util_1.expectArraysClose(tf.equalStrict(a, b), [0, 0, 0]);
    });
    // Tensor2D:
    it('Tensor2D - int32', function () {
        var a = tf.tensor2d([[1, 4, 5], [8, 9, 12]], [2, 3], 'int32');
        var b = tf.tensor2d([[2, 3, 6], [7, 10, 11]], [2, 3], 'int32');
        test_util_1.expectArraysClose(tf.equalStrict(a, b), [0, 0, 0, 0, 0, 0]);
        a = tf.tensor2d([[0, 0], [1, 1]], [2, 2], 'int32');
        b = tf.tensor2d([[0, 0], [1, 1]], [2, 2], 'int32');
        test_util_1.expectArraysClose(tf.equalStrict(a, b), [1, 1, 1, 1]);
    });
    it('Tensor2D - float32', function () {
        var a = tf.tensor2d([[1.1, 4.1, 5.1], [8.1, 9.1, 12.1]], [2, 3], 'float32');
        var b = tf.tensor2d([[2.1, 4.1, 5.1], [7.1, 10.1, 11.1]], [2, 3], 'float32');
        test_util_1.expectArraysClose(tf.equalStrict(a, b), [0, 1, 1, 0, 0, 0]);
        a = tf.tensor2d([[0.2, 0.2], [1.2, 1.2]], [2, 2], 'float32');
        b = tf.tensor2d([[0.2, 0.2], [1.2, 1.2]], [2, 2], 'float32');
        test_util_1.expectArraysClose(tf.equalStrict(a, b), [1, 1, 1, 1]);
    });
    it('mismatch Tensor2D shapes - int32', function () {
        var a = tf.tensor2d([[3], [7]], [2, 1], 'int32');
        var b = tf.tensor2d([[2, 3, 4], [7, 8, 9]], [2, 3], 'int32');
        var f = function () {
            tf.equalStrict(a, b);
        };
        expect(f).toThrowError();
    });
    it('mismatch Tensor2D shapes - float32', function () {
        var a = tf.tensor2d([[1.1], [7.1]], [2, 1], 'float32');
        var b = tf.tensor2d([[0.1, 1.1, 2.1], [7.1, 8.1, 9.1]], [2, 3], 'float32');
        var f = function () {
            tf.equalStrict(a, b);
        };
        expect(f).toThrowError();
    });
    it('NaNs in Tensor2D - float32', function () {
        var a = tf.tensor2d([[1.1, NaN], [1.1, NaN]], [2, 2], 'float32');
        var b = tf.tensor2d([[0.1, NaN], [1.1, NaN]], [2, 2], 'float32');
        test_util_1.expectArraysClose(tf.equalStrict(a, b), [0, 0, 1, 0]);
    });
    // Tensor3D:
    it('Tensor3D - int32', function () {
        var a = tf.tensor3d([[[1], [4], [5]], [[8], [9], [12]]], [2, 3, 1], 'int32');
        var b = tf.tensor3d([[[2], [3], [6]], [[7], [10], [12]]], [2, 3, 1], 'int32');
        test_util_1.expectArraysClose(tf.equalStrict(a, b), [0, 0, 0, 0, 0, 1]);
        a = tf.tensor3d([[[0], [0], [0]], [[1], [1], [1]]], [2, 3, 1], 'int32');
        b = tf.tensor3d([[[0], [0], [0]], [[1], [1], [1]]], [2, 3, 1], 'int32');
        test_util_1.expectArraysClose(tf.equalStrict(a, b), [1, 1, 1, 1, 1, 1]);
    });
    it('Tensor3D - float32', function () {
        var a = tf.tensor3d([[[1.1], [4.1], [5.1]], [[8.1], [9.1], [12.1]]], [2, 3, 1], 'float32');
        var b = tf.tensor3d([[[2.1], [3.1], [6.1]], [[7.1], [10.1], [12.1]]], [2, 3, 1], 'float32');
        test_util_1.expectArraysClose(tf.equalStrict(a, b), [0, 0, 0, 0, 0, 1]);
        a = tf.tensor3d([[[0.1], [0.1], [0.1]], [[1.1], [1.1], [1.1]]], [2, 3, 1], 'float32');
        b = tf.tensor3d([[[0.1], [0.1], [0.1]], [[1.1], [1.1], [1.1]]], [2, 3, 1], 'float32');
        test_util_1.expectArraysClose(tf.equalStrict(a, b), [1, 1, 1, 1, 1, 1]);
    });
    it('mismatch Tensor3D shapes - int32', function () {
        var a = tf.tensor3d([[[1, 0], [2, 3], [4, 5]], [[6, 7], [9, 8], [10, 11]]], [2, 3, 2], 'int32');
        var b = tf.tensor3d([[[1], [2], [3]], [[7], [10], [9]]], [2, 3, 1], 'int32');
        var f = function () {
            tf.equalStrict(a, b);
        };
        expect(f).toThrowError();
    });
    it('mismatch Tensor3D shapes - float32', function () {
        var a = tf.tensor3d([
            [[1.1, 0.1], [2.1, 3.1], [4.1, 5.1]],
            [[6.1, 7.1], [9.1, 8.1], [10.1, 11.1]]
        ], [2, 3, 2], 'float32');
        var b = tf.tensor3d([[[1.1], [2.1], [3.1]], [[7.1], [10.1], [9.1]]], [2, 3, 1], 'float32');
        var f = function () {
            tf.equalStrict(a, b);
        };
        expect(f).toThrowError();
    });
    it('NaNs in Tensor3D - float32', function () {
        var a = tf.tensor3d([[[1.1], [NaN], [1.1]], [[0.1], [0.1], [0.1]]], [2, 3, 1], 'float32');
        var b = tf.tensor3d([[[0.1], [0.1], [1.1]], [[1.1], [0.1], [NaN]]], [2, 3, 1], 'float32');
        test_util_1.expectArraysClose(tf.equalStrict(a, b), [0, 0, 1, 0, 1, 0]);
    });
    // Tensor4D:
    it('Tensor4D - int32', function () {
        var a = tf.tensor4d([1, 4, 5, 8], [2, 2, 1, 1], 'int32');
        var b = tf.tensor4d([2, 3, 6, 8], [2, 2, 1, 1], 'int32');
        test_util_1.expectArraysClose(tf.equalStrict(a, b), [0, 0, 0, 1]);
        a = tf.tensor4d([0, 1, 2, 3], [2, 2, 1, 1], 'int32');
        b = tf.tensor4d([0, 1, 2, 3], [2, 2, 1, 1], 'int32');
        test_util_1.expectArraysClose(tf.equalStrict(a, b), [1, 1, 1, 1]);
        a = tf.tensor4d([1, 1, 1, 1], [2, 2, 1, 1], 'int32');
        b = tf.tensor4d([2, 2, 2, 2], [2, 2, 1, 1], 'int32');
        test_util_1.expectArraysClose(tf.equalStrict(a, b), [0, 0, 0, 0]);
    });
    it('Tensor4D - float32', function () {
        var a = tf.tensor4d([1.1, 4.1, 5.1, 8.1], [2, 2, 1, 1], 'float32');
        var b = tf.tensor4d([2.1, 3.1, 6.1, 8.1], [2, 2, 1, 1], 'float32');
        test_util_1.expectArraysClose(tf.equalStrict(a, b), [0, 0, 0, 1]);
        a = tf.tensor4d([0.1, 1.1, 2.2, 3.3], [2, 2, 1, 1], 'float32');
        b = tf.tensor4d([0.1, 1.1, 2.2, 3.3], [2, 2, 1, 1], 'float32');
        test_util_1.expectArraysClose(tf.equalStrict(a, b), [1, 1, 1, 1]);
        a = tf.tensor4d([0.1, 0.1, 0.1, 0.1], [2, 2, 1, 1], 'float32');
        b = tf.tensor4d([1.1, 1.1, 1.1, 1.1], [2, 2, 1, 1], 'float32');
        test_util_1.expectArraysClose(tf.equalStrict(a, b), [0, 0, 0, 0]);
    });
    it('mismatch Tensor4D shapes - int32', function () {
        var a = tf.tensor4d([1, 2, 5, 9], [2, 2, 1, 1], 'int32');
        var b = tf.tensor4d([[[[1, 2]], [[3, 4]]], [[[5, 6]], [[7, 8]]]], [2, 2, 1, 2], 'int32');
        var f = function () {
            tf.equalStrict(a, b);
        };
        expect(f).toThrowError();
    });
    it('mismatch Tensor4D shapes - float32', function () {
        var a = tf.tensor4d([1.1, 2.1, 5.1, 9.1], [2, 2, 1, 1], 'float32');
        var b = tf.tensor4d([[[[1.1, 2.1]], [[3.1, 4.1]]], [[[5.1, 6.1]], [[7.1, 8.1]]]], [2, 2, 1, 2], 'float32');
        var f = function () {
            tf.equalStrict(a, b);
        };
        expect(f).toThrowError();
    });
    it('NaNs in Tensor4D - float32', function () {
        var a = tf.tensor4d([1.1, NaN, 1.1, 0.1], [2, 2, 1, 1], 'float32');
        var b = tf.tensor4d([0.1, 1.1, 1.1, NaN], [2, 2, 1, 1], 'float32');
        test_util_1.expectArraysClose(tf.equalStrict(a, b), [0, 0, 1, 0]);
    });
    it('accepts a tensor-like object', function () {
        var a = [1, 4, 5];
        var b = [2, 3, 5];
        test_util_1.expectArraysClose(tf.equalStrict(a, b), [0, 0, 1]);
    });
});
jasmine_util_1.describeWithFlags('notEqual', test_util_1.ALL_ENVS, function () {
    it('Tensor1D - int32', function () {
        var a = tf.tensor1d([1, 4, 5], 'int32');
        var b = tf.tensor1d([2, 3, 5], 'int32');
        test_util_1.expectArraysClose(tf.notEqual(a, b), [1, 1, 0]);
        a = tf.tensor1d([2, 2, 2], 'int32');
        b = tf.tensor1d([2, 2, 2], 'int32');
        test_util_1.expectArraysClose(tf.notEqual(a, b), [0, 0, 0]);
        a = tf.tensor1d([0, 0], 'int32');
        b = tf.tensor1d([3, 3], 'int32');
        test_util_1.expectArraysClose(tf.notEqual(a, b), [1, 1]);
    });
    it('Tensor1D - float32', function () {
        var a = tf.tensor1d([1.1, 4.1, 5.1], 'float32');
        var b = tf.tensor1d([2.2, 3.2, 5.1], 'float32');
        test_util_1.expectArraysClose(tf.notEqual(a, b), [1, 1, 0]);
        a = tf.tensor1d([2.31, 2.31, 2.31], 'float32');
        b = tf.tensor1d([2.31, 2.31, 2.31], 'float32');
        test_util_1.expectArraysClose(tf.notEqual(a, b), [0, 0, 0]);
        a = tf.tensor1d([0.45, 0.123], 'float32');
        b = tf.tensor1d([3.123, 3.321], 'float32');
        test_util_1.expectArraysClose(tf.notEqual(a, b), [1, 1]);
    });
    it('upcasts when dtypes dont match', function () {
        var a = [1.1, 4.1, 5];
        var b = [2.2, 3.2, 5];
        var res = tf.notEqual(tf.tensor(a, [3], 'float32'), tf.tensor(b, [3], 'int32'));
        expect(res.dtype).toBe('bool');
        expect(res.shape).toEqual([3]);
        test_util_1.expectArraysClose(res, [1, 1, 0]);
        res =
            tf.notEqual(tf.tensor(a, [3], 'int32'), tf.tensor(b, [3], 'bool'));
        expect(res.dtype).toBe('bool');
        expect(res.shape).toEqual([3]);
        test_util_1.expectArraysClose(res, [0, 1, 1]);
    });
    it('TensorLike', function () {
        var a = [1.1, 4.1, 5.1];
        var b = [2.2, 3.2, 5.1];
        test_util_1.expectArraysClose(tf.notEqual(a, b), [1, 1, 0]);
    });
    it('TensorLike Chained', function () {
        var a = tf.tensor1d([1.1, 4.1, 5.1], 'float32');
        var b = [2.2, 3.2, 5.1];
        test_util_1.expectArraysClose(a.notEqual(b), [1, 1, 0]);
    });
    it('mismatched Tensor1D shapes - int32', function () {
        var a = tf.tensor1d([1, 2], 'int32');
        var b = tf.tensor1d([1, 2, 3], 'int32');
        var f = function () {
            tf.notEqual(a, b);
        };
        expect(f).toThrowError();
    });
    it('mismatched Tensor1D shapes - float32', function () {
        var a = tf.tensor1d([1.1, 2.1], 'float32');
        var b = tf.tensor1d([1.1, 2.1, 3.1], 'float32');
        var f = function () {
            tf.notEqual(a, b);
        };
        expect(f).toThrowError();
    });
    it('NaNs in Tensor1D - float32', function () {
        var a = tf.tensor1d([1.1, NaN, 2.1], 'float32');
        var b = tf.tensor1d([2.1, 3.1, NaN], 'float32');
        test_util_1.expectArraysClose(tf.notEqual(a, b), [1, 1, 1]);
    });
    it('works with NaNs', function () {
        var a = tf.tensor1d([2, 5, NaN]);
        var b = tf.tensor1d([4, 5, -1]);
        var res = tf.notEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysEqual(res, [1, 0, 1]);
    });
    it('scalar and 1D broadcast', function () {
        var a = tf.scalar(2);
        var b = tf.tensor1d([1, 2, 3, 4, 5, 2]);
        var res = tf.notEqual(a, b);
        expect(res.dtype).toBe('bool');
        expect(res.shape).toEqual([6]);
        test_util_1.expectArraysEqual(res, [1, 0, 1, 1, 1, 0]);
    });
    // Tensor2D:
    it('Tensor2D - int32', function () {
        var a = tf.tensor2d([[1, 4, 5], [8, 9, 12]], [2, 3], 'int32');
        var b = tf.tensor2d([[2, 3, 6], [7, 10, 11]], [2, 3], 'int32');
        test_util_1.expectArraysClose(tf.notEqual(a, b), [1, 1, 1, 1, 1, 1]);
        a = tf.tensor2d([[0, 0], [1, 1]], [2, 2], 'int32');
        b = tf.tensor2d([[0, 0], [1, 1]], [2, 2], 'int32');
        test_util_1.expectArraysClose(tf.notEqual(a, b), [0, 0, 0, 0]);
    });
    it('Tensor2D - float32', function () {
        var a = tf.tensor2d([[1.1, 4.1, 5.1], [8.1, 9.1, 12.1]], [2, 3], 'float32');
        var b = tf.tensor2d([[2.1, 4.1, 5.1], [7.1, 10.1, 11.1]], [2, 3], 'float32');
        test_util_1.expectArraysClose(tf.notEqual(a, b), [1, 0, 0, 1, 1, 1]);
        a = tf.tensor2d([[0.2, 0.2], [1.2, 1.2]], [2, 2], 'float32');
        b = tf.tensor2d([[0.2, 0.2], [1.2, 1.2]], [2, 2], 'float32');
        test_util_1.expectArraysClose(tf.notEqual(a, b), [0, 0, 0, 0]);
    });
    it('broadcasting Tensor2D shapes - int32', function () {
        var a = tf.tensor2d([[3], [7]], [2, 1], 'int32');
        var b = tf.tensor2d([[2, 3, 4], [7, 8, 9]], [2, 3], 'int32');
        test_util_1.expectArraysClose(tf.notEqual(a, b), [1, 0, 1, 0, 1, 1]);
    });
    it('broadcasting Tensor2D shapes - float32', function () {
        var a = tf.tensor2d([[1.1], [7.1]], [2, 1], 'float32');
        var b = tf.tensor2d([[0.1, 1.1, 2.1], [7.1, 8.1, 9.1]], [2, 3], 'float32');
        test_util_1.expectArraysClose(tf.notEqual(a, b), [1, 0, 1, 0, 1, 1]);
    });
    it('NaNs in Tensor2D - float32', function () {
        var a = tf.tensor2d([[1.1, NaN], [1.1, NaN]], [2, 2], 'float32');
        var b = tf.tensor2d([[0.1, NaN], [1.1, NaN]], [2, 2], 'float32');
        test_util_1.expectArraysClose(tf.notEqual(a, b), [1, 1, 0, 1]);
    });
    it('2D and scalar broadcast', function () {
        var a = tf.tensor2d([1, 2, 3, 2, 5, 6], [2, 3]);
        var b = tf.scalar(2);
        var res = tf.notEqual(a, b);
        expect(res.dtype).toBe('bool');
        expect(res.shape).toEqual([2, 3]);
        test_util_1.expectArraysEqual(res, [1, 0, 1, 0, 1, 1]);
    });
    it('2D and 2D broadcast each with 1 dim', function () {
        var a = tf.tensor2d([1, 2, 5], [1, 3]);
        var b = tf.tensor2d([5, 1], [2, 1]);
        var res = tf.notEqual(a, b);
        expect(res.dtype).toBe('bool');
        expect(res.shape).toEqual([2, 3]);
        test_util_1.expectArraysEqual(res, [1, 1, 0, 0, 1, 1]);
    });
    // Tensor3D:
    it('Tensor3D - int32', function () {
        var a = tf.tensor3d([[[1], [4], [5]], [[8], [9], [12]]], [2, 3, 1], 'int32');
        var b = tf.tensor3d([[[2], [3], [6]], [[7], [10], [12]]], [2, 3, 1], 'int32');
        test_util_1.expectArraysClose(tf.notEqual(a, b), [1, 1, 1, 1, 1, 0]);
        a = tf.tensor3d([[[0], [0], [0]], [[1], [1], [1]]], [2, 3, 1], 'int32');
        b = tf.tensor3d([[[0], [0], [0]], [[1], [1], [1]]], [2, 3, 1], 'int32');
        test_util_1.expectArraysClose(tf.notEqual(a, b), [0, 0, 0, 0, 0, 0]);
    });
    it('Tensor3D - float32', function () {
        var a = tf.tensor3d([[[1.1], [4.1], [5.1]], [[8.1], [9.1], [12.1]]], [2, 3, 1], 'float32');
        var b = tf.tensor3d([[[2.1], [3.1], [6.1]], [[7.1], [10.1], [12.1]]], [2, 3, 1], 'float32');
        test_util_1.expectArraysClose(tf.notEqual(a, b), [1, 1, 1, 1, 1, 0]);
        a = tf.tensor3d([[[0.1], [0.1], [0.1]], [[1.1], [1.1], [1.1]]], [2, 3, 1], 'float32');
        b = tf.tensor3d([[[0.1], [0.1], [0.1]], [[1.1], [1.1], [1.1]]], [2, 3, 1], 'float32');
        test_util_1.expectArraysClose(tf.notEqual(a, b), [0, 0, 0, 0, 0, 0]);
    });
    it('broadcasting Tensor3D shapes - int32', function () {
        var a = tf.tensor3d([[[1, 0], [2, 3], [4, 5]], [[6, 7], [9, 8], [10, 11]]], [2, 3, 2], 'int32');
        var b = tf.tensor3d([[[1], [2], [3]], [[7], [10], [9]]], [2, 3, 1], 'int32');
        test_util_1.expectArraysClose(tf.notEqual(a, b), [0, 1, 0, 1, 1, 1, 1, 0, 1, 1, 1, 1]);
    });
    it('broadcasting Tensor3D shapes - float32', function () {
        var a = tf.tensor3d([
            [[1.1, 0.1], [2.1, 3.1], [4.1, 5.1]],
            [[6.1, 7.1], [9.1, 8.1], [10.1, 11.1]]
        ], [2, 3, 2], 'float32');
        var b = tf.tensor3d([[[1.1], [2.1], [3.1]], [[7.1], [10.1], [9.1]]], [2, 3, 1], 'float32');
        test_util_1.expectArraysClose(tf.notEqual(a, b), [0, 1, 0, 1, 1, 1, 1, 0, 1, 1, 1, 1]);
    });
    it('NaNs in Tensor3D - float32', function () {
        var a = tf.tensor3d([[[1.1], [NaN], [1.1]], [[0.1], [0.1], [0.1]]], [2, 3, 1], 'float32');
        var b = tf.tensor3d([[[0.1], [0.1], [1.1]], [[1.1], [0.1], [NaN]]], [2, 3, 1], 'float32');
        test_util_1.expectArraysClose(tf.notEqual(a, b), [1, 1, 0, 1, 0, 1]);
    });
    it('3D and scalar', function () {
        var a = tf.tensor3d([1, 2, 3, 4, 5, -1], [2, 3, 1]);
        var b = tf.scalar(-1);
        var res = tf.notEqual(a, b);
        expect(res.dtype).toBe('bool');
        expect(res.shape).toEqual([2, 3, 1]);
        test_util_1.expectArraysEqual(res, [1, 1, 1, 1, 1, 0]);
    });
    // Tensor4D:
    it('Tensor4D - int32', function () {
        var a = tf.tensor4d([1, 4, 5, 8], [2, 2, 1, 1], 'int32');
        var b = tf.tensor4d([2, 3, 6, 8], [2, 2, 1, 1], 'int32');
        test_util_1.expectArraysClose(tf.notEqual(a, b), [1, 1, 1, 0]);
        a = tf.tensor4d([0, 1, 2, 3], [2, 2, 1, 1], 'int32');
        b = tf.tensor4d([0, 1, 2, 3], [2, 2, 1, 1], 'int32');
        test_util_1.expectArraysClose(tf.notEqual(a, b), [0, 0, 0, 0]);
        a = tf.tensor4d([1, 1, 1, 1], [2, 2, 1, 1], 'int32');
        b = tf.tensor4d([2, 2, 2, 2], [2, 2, 1, 1], 'int32');
        test_util_1.expectArraysClose(tf.notEqual(a, b), [1, 1, 1, 1]);
    });
    it('Tensor4D - float32', function () {
        var a = tf.tensor4d([1.1, 4.1, 5.1, 8.1], [2, 2, 1, 1], 'float32');
        var b = tf.tensor4d([2.1, 3.1, 6.1, 8.1], [2, 2, 1, 1], 'float32');
        test_util_1.expectArraysClose(tf.notEqual(a, b), [1, 1, 1, 0]);
        a = tf.tensor4d([0.1, 1.1, 2.2, 3.3], [2, 2, 1, 1], 'float32');
        b = tf.tensor4d([0.1, 1.1, 2.2, 3.3], [2, 2, 1, 1], 'float32');
        test_util_1.expectArraysClose(tf.notEqual(a, b), [0, 0, 0, 0]);
        a = tf.tensor4d([0.1, 0.1, 0.1, 0.1], [2, 2, 1, 1], 'float32');
        b = tf.tensor4d([1.1, 1.1, 1.1, 1.1], [2, 2, 1, 1], 'float32');
        test_util_1.expectArraysClose(tf.notEqual(a, b), [1, 1, 1, 1]);
    });
    it('broadcasting Tensor4D shapes - int32', function () {
        var a = tf.tensor4d([1, 2, 5, 9], [2, 2, 1, 1], 'int32');
        var b = tf.tensor4d([[[[1, 2]], [[3, 4]]], [[[5, 6]], [[7, 8]]]], [2, 2, 1, 2], 'int32');
        test_util_1.expectArraysClose(tf.notEqual(a, b), [0, 1, 1, 1, 0, 1, 1, 1]);
    });
    it('broadcasting Tensor4D shapes - float32', function () {
        var a = tf.tensor4d([1.1, 2.1, 5.1, 9.1], [2, 2, 1, 1], 'float32');
        var b = tf.tensor4d([[[[1.1, 2.1]], [[3.1, 4.1]]], [[[5.1, 6.1]], [[7.1, 8.1]]]], [2, 2, 1, 2], 'float32');
        test_util_1.expectArraysClose(tf.notEqual(a, b), [0, 1, 1, 1, 0, 1, 1, 1]);
    });
    it('NaNs in Tensor4D - float32', function () {
        var a = tf.tensor4d([1.1, NaN, 1.1, 0.1], [2, 2, 1, 1], 'float32');
        var b = tf.tensor4d([0.1, 1.1, 1.1, NaN], [2, 2, 1, 1], 'float32');
        test_util_1.expectArraysClose(tf.notEqual(a, b), [1, 1, 0, 1]);
    });
    it('throws when passed a as a non-tensor', function () {
        expect(function () { return tf.notEqual({}, tf.scalar(1)); })
            .toThrowError(/Argument 'a' passed to 'notEqual' must be a Tensor/);
    });
    it('throws when passed b as a non-tensor', function () {
        expect(function () { return tf.notEqual(tf.scalar(1), {}); })
            .toThrowError(/Argument 'b' passed to 'notEqual' must be a Tensor/);
    });
    it('accepts a tensor-like object', function () {
        var a = tf.tensor1d([1, 4, 5], 'int32');
        var b = tf.tensor1d([2, 3, 5], 'int32');
        test_util_1.expectArraysClose(tf.notEqual(a, b), [1, 1, 0]);
    });
});
jasmine_util_1.describeWithFlags('notEqualStrict', test_util_1.ALL_ENVS, function () {
    it('Tensor1D - int32', function () {
        var a = tf.tensor1d([1, 4, 5], 'int32');
        var b = tf.tensor1d([2, 3, 5], 'int32');
        test_util_1.expectArraysClose(tf.notEqualStrict(a, b), [1, 1, 0]);
        a = tf.tensor1d([2, 2, 2], 'int32');
        b = tf.tensor1d([2, 2, 2], 'int32');
        test_util_1.expectArraysClose(tf.notEqualStrict(a, b), [0, 0, 0]);
        a = tf.tensor1d([0, 0], 'int32');
        b = tf.tensor1d([3, 3], 'int32');
        test_util_1.expectArraysClose(tf.notEqualStrict(a, b), [1, 1]);
    });
    it('Tensor1D - float32', function () {
        var a = tf.tensor1d([1.1, 4.1, 5.1], 'float32');
        var b = tf.tensor1d([2.2, 3.2, 5.1], 'float32');
        test_util_1.expectArraysClose(tf.notEqualStrict(a, b), [1, 1, 0]);
        a = tf.tensor1d([2.31, 2.31, 2.31], 'float32');
        b = tf.tensor1d([2.31, 2.31, 2.31], 'float32');
        test_util_1.expectArraysClose(tf.notEqualStrict(a, b), [0, 0, 0]);
        a = tf.tensor1d([0.45, 0.123], 'float32');
        b = tf.tensor1d([3.123, 3.321], 'float32');
        test_util_1.expectArraysClose(tf.notEqualStrict(a, b), [1, 1]);
    });
    it('mismatched Tensor1D shapes - int32', function () {
        var a = tf.tensor1d([1, 2], 'int32');
        var b = tf.tensor1d([1, 2, 3], 'int32');
        var f = function () {
            tf.notEqualStrict(a, b);
        };
        expect(f).toThrowError();
    });
    it('mismatched Tensor1D shapes - float32', function () {
        var a = tf.tensor1d([1.1, 2.1], 'float32');
        var b = tf.tensor1d([1.1, 2.1, 3.1], 'float32');
        var f = function () {
            tf.notEqualStrict(a, b);
        };
        expect(f).toThrowError();
    });
    it('NaNs in Tensor1D - float32', function () {
        var a = tf.tensor1d([1.1, NaN, 2.1], 'float32');
        var b = tf.tensor1d([2.1, 3.1, NaN], 'float32');
        test_util_1.expectArraysClose(tf.notEqualStrict(a, b), [1, 1, 1]);
    });
    it('strict version throws when x and y are different shape', function () {
        var a = tf.tensor1d([2]);
        var b = tf.tensor1d([4, 2, -1]);
        expect(function () { return tf.notEqualStrict(a, b); }).toThrowError();
        expect(function () { return tf.notEqualStrict(b, a); }).toThrowError();
    });
    // Tensor2D:
    it('Tensor2D - int32', function () {
        var a = tf.tensor2d([[1, 4, 5], [8, 9, 12]], [2, 3], 'int32');
        var b = tf.tensor2d([[2, 3, 6], [7, 10, 11]], [2, 3], 'int32');
        test_util_1.expectArraysClose(tf.notEqualStrict(a, b), [1, 1, 1, 1, 1, 1]);
        a = tf.tensor2d([[0, 0], [1, 1]], [2, 2], 'int32');
        b = tf.tensor2d([[0, 0], [1, 1]], [2, 2], 'int32');
        test_util_1.expectArraysClose(tf.notEqualStrict(a, b), [0, 0, 0, 0]);
    });
    it('Tensor2D - float32', function () {
        var a = tf.tensor2d([[1.1, 4.1, 5.1], [8.1, 9.1, 12.1]], [2, 3], 'float32');
        var b = tf.tensor2d([[2.1, 4.1, 5.1], [7.1, 10.1, 11.1]], [2, 3], 'float32');
        test_util_1.expectArraysClose(tf.notEqualStrict(a, b), [1, 0, 0, 1, 1, 1]);
        a = tf.tensor2d([[0.2, 0.2], [1.2, 1.2]], [2, 2], 'float32');
        b = tf.tensor2d([[0.2, 0.2], [1.2, 1.2]], [2, 2], 'float32');
        test_util_1.expectArraysClose(tf.notEqualStrict(a, b), [0, 0, 0, 0]);
    });
    it('mismatch Tensor2D shapes - int32', function () {
        var a = tf.tensor2d([[3], [7]], [2, 1], 'int32');
        var b = tf.tensor2d([[2, 3, 4], [7, 8, 9]], [2, 3], 'int32');
        var f = function () {
            tf.notEqualStrict(a, b);
        };
        expect(f).toThrowError();
    });
    it('mismatch Tensor2D shapes - float32', function () {
        var a = tf.tensor2d([[1.1], [7.1]], [2, 1], 'float32');
        var b = tf.tensor2d([[0.1, 1.1, 2.1], [7.1, 8.1, 9.1]], [2, 3], 'float32');
        var f = function () {
            tf.notEqualStrict(a, b);
        };
        expect(f).toThrowError();
    });
    it('NaNs in Tensor2D - float32', function () {
        var a = tf.tensor2d([[1.1, NaN], [1.1, NaN]], [2, 2], 'float32');
        var b = tf.tensor2d([[0.1, NaN], [1.1, NaN]], [2, 2], 'float32');
        test_util_1.expectArraysClose(tf.notEqualStrict(a, b), [1, 1, 0, 1]);
    });
    // Tensor3D:
    it('Tensor3D - int32', function () {
        var a = tf.tensor3d([[[1], [4], [5]], [[8], [9], [12]]], [2, 3, 1], 'int32');
        var b = tf.tensor3d([[[2], [3], [6]], [[7], [10], [12]]], [2, 3, 1], 'int32');
        test_util_1.expectArraysClose(tf.notEqualStrict(a, b), [1, 1, 1, 1, 1, 0]);
        a = tf.tensor3d([[[0], [0], [0]], [[1], [1], [1]]], [2, 3, 1], 'int32');
        b = tf.tensor3d([[[0], [0], [0]], [[1], [1], [1]]], [2, 3, 1], 'int32');
        test_util_1.expectArraysClose(tf.notEqualStrict(a, b), [0, 0, 0, 0, 0, 0]);
    });
    it('Tensor3D - float32', function () {
        var a = tf.tensor3d([[[1.1], [4.1], [5.1]], [[8.1], [9.1], [12.1]]], [2, 3, 1], 'float32');
        var b = tf.tensor3d([[[2.1], [3.1], [6.1]], [[7.1], [10.1], [12.1]]], [2, 3, 1], 'float32');
        test_util_1.expectArraysClose(tf.notEqualStrict(a, b), [1, 1, 1, 1, 1, 0]);
        a = tf.tensor3d([[[0.1], [0.1], [0.1]], [[1.1], [1.1], [1.1]]], [2, 3, 1], 'float32');
        b = tf.tensor3d([[[0.1], [0.1], [0.1]], [[1.1], [1.1], [1.1]]], [2, 3, 1], 'float32');
        test_util_1.expectArraysClose(tf.notEqualStrict(a, b), [0, 0, 0, 0, 0, 0]);
    });
    it('mismatch Tensor3D shapes - int32', function () {
        var a = tf.tensor3d([[[1, 0], [2, 3], [4, 5]], [[6, 7], [9, 8], [10, 11]]], [2, 3, 2], 'int32');
        var b = tf.tensor3d([[[1], [2], [3]], [[7], [10], [9]]], [2, 3, 1], 'int32');
        var f = function () {
            tf.notEqualStrict(a, b);
        };
        expect(f).toThrowError();
    });
    it('mismatch Tensor3D shapes - float32', function () {
        var a = tf.tensor3d([
            [[1.1, 0.1], [2.1, 3.1], [4.1, 5.1]],
            [[6.1, 7.1], [9.1, 8.1], [10.1, 11.1]]
        ], [2, 3, 2], 'float32');
        var b = tf.tensor3d([[[1.1], [2.1], [3.1]], [[7.1], [10.1], [9.1]]], [2, 3, 1], 'float32');
        var f = function () {
            tf.notEqualStrict(a, b);
        };
        expect(f).toThrowError();
    });
    it('NaNs in Tensor3D - float32', function () {
        var a = tf.tensor3d([[[1.1], [NaN], [1.1]], [[0.1], [0.1], [0.1]]], [2, 3, 1], 'float32');
        var b = tf.tensor3d([[[0.1], [0.1], [1.1]], [[1.1], [0.1], [NaN]]], [2, 3, 1], 'float32');
        test_util_1.expectArraysClose(tf.notEqualStrict(a, b), [1, 1, 0, 1, 0, 1]);
    });
    // Tensor4D:
    it('Tensor4D - int32', function () {
        var a = tf.tensor4d([1, 4, 5, 8], [2, 2, 1, 1], 'int32');
        var b = tf.tensor4d([2, 3, 6, 8], [2, 2, 1, 1], 'int32');
        test_util_1.expectArraysClose(tf.notEqualStrict(a, b), [1, 1, 1, 0]);
        a = tf.tensor4d([0, 1, 2, 3], [2, 2, 1, 1], 'int32');
        b = tf.tensor4d([0, 1, 2, 3], [2, 2, 1, 1], 'int32');
        test_util_1.expectArraysClose(tf.notEqualStrict(a, b), [0, 0, 0, 0]);
        a = tf.tensor4d([1, 1, 1, 1], [2, 2, 1, 1], 'int32');
        b = tf.tensor4d([2, 2, 2, 2], [2, 2, 1, 1], 'int32');
        test_util_1.expectArraysClose(tf.notEqualStrict(a, b), [1, 1, 1, 1]);
    });
    it('Tensor4D - float32', function () {
        var a = tf.tensor4d([1.1, 4.1, 5.1, 8.1], [2, 2, 1, 1], 'float32');
        var b = tf.tensor4d([2.1, 3.1, 6.1, 8.1], [2, 2, 1, 1], 'float32');
        test_util_1.expectArraysClose(tf.notEqualStrict(a, b), [1, 1, 1, 0]);
        a = tf.tensor4d([0.1, 1.1, 2.2, 3.3], [2, 2, 1, 1], 'float32');
        b = tf.tensor4d([0.1, 1.1, 2.2, 3.3], [2, 2, 1, 1], 'float32');
        test_util_1.expectArraysClose(tf.notEqualStrict(a, b), [0, 0, 0, 0]);
        a = tf.tensor4d([0.1, 0.1, 0.1, 0.1], [2, 2, 1, 1], 'float32');
        b = tf.tensor4d([1.1, 1.1, 1.1, 1.1], [2, 2, 1, 1], 'float32');
        test_util_1.expectArraysClose(tf.notEqualStrict(a, b), [1, 1, 1, 1]);
    });
    it('mismatch Tensor4D shapes - int32', function () {
        var a = tf.tensor4d([1, 2, 5, 9], [2, 2, 1, 1], 'int32');
        var b = tf.tensor4d([[[[1, 2]], [[3, 4]]], [[[5, 6]], [[7, 8]]]], [2, 2, 1, 2], 'int32');
        var f = function () {
            tf.notEqualStrict(a, b);
        };
        expect(f).toThrowError();
    });
    it('mismatch Tensor4D shapes - float32', function () {
        var a = tf.tensor4d([1.1, 2.1, 5.1, 9.1], [2, 2, 1, 1], 'float32');
        var b = tf.tensor4d([[[[1.1, 2.1]], [[3.1, 4.1]]], [[[5.1, 6.1]], [[7.1, 8.1]]]], [2, 2, 1, 2], 'float32');
        var f = function () {
            tf.notEqualStrict(a, b);
        };
        expect(f).toThrowError();
    });
    it('NaNs in Tensor4D - float32', function () {
        var a = tf.tensor4d([1.1, NaN, 1.1, 0.1], [2, 2, 1, 1], 'float32');
        var b = tf.tensor4d([0.1, 1.1, 1.1, NaN], [2, 2, 1, 1], 'float32');
        test_util_1.expectArraysClose(tf.notEqualStrict(a, b), [1, 1, 0, 1]);
    });
    it('accepts a tensor-like object', function () {
        var a = [1, 4, 5];
        var b = [2, 3, 5];
        test_util_1.expectArraysClose(tf.notEqualStrict(a, b), [1, 1, 0]);
    });
});
jasmine_util_1.describeWithFlags('less', test_util_1.ALL_ENVS, function () {
    it('Tensor1D - int32', function () {
        var a = tf.tensor1d([1, 4, 5], 'int32');
        var b = tf.tensor1d([2, 3, 5], 'int32');
        var res = tf.less(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 0]);
        a = tf.tensor1d([2, 2, 2], 'int32');
        b = tf.tensor1d([2, 2, 2], 'int32');
        res = tf.less(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 0, 0]);
        a = tf.tensor1d([0, 0], 'int32');
        b = tf.tensor1d([3, 3], 'int32');
        res = tf.less(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 1]);
    });
    it('Tensor1D - float32', function () {
        var a = tf.tensor1d([1.1, 4.1, 5.1], 'float32');
        var b = tf.tensor1d([2.2, 3.2, 5.1], 'float32');
        var res = tf.less(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 0]);
        a = tf.tensor1d([2.31, 2.31, 2.31], 'float32');
        b = tf.tensor1d([2.31, 2.31, 2.31], 'float32');
        res = tf.less(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 0, 0]);
        a = tf.tensor1d([0.45, 0.123], 'float32');
        b = tf.tensor1d([3.123, 3.321], 'float32');
        res = tf.less(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 1]);
    });
    it('TensorLike', function () {
        var a = [1.1, 4.1, 5.1];
        var b = [2.2, 3.2, 5.1];
        var res = tf.less(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 0]);
    });
    it('TensorLike Chained', function () {
        var a = tf.tensor1d([1.1, 4.1, 5.1], 'float32');
        var b = [2.2, 3.2, 5.1];
        var res = a.less(b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 0]);
    });
    it('upcasts when dtypes dont match', function () {
        var a = [1.1, 4.1, 5.2];
        var b = [2.2, 3.2, 5.1];
        var res = tf.less(tf.tensor(a, [3], 'float32'), tf.tensor(b, [3], 'int32'));
        expect(res.dtype).toBe('bool');
        expect(res.shape).toEqual([3]);
        test_util_1.expectArraysClose(res, [1, 0, 0]);
        res =
            tf.less(tf.tensor(a, [3], 'int32'), tf.tensor(b, [3], 'bool'));
        expect(res.dtype).toBe('bool');
        expect(res.shape).toEqual([3]);
        test_util_1.expectArraysClose(res, [0, 0, 0]);
    });
    it('mismatched Tensor1D shapes - int32', function () {
        var a = tf.tensor1d([1, 2], 'int32');
        var b = tf.tensor1d([1, 2, 3], 'int32');
        var f = function () {
            tf.less(a, b);
        };
        expect(f).toThrowError();
    });
    it('mismatched Tensor1D shapes - float32', function () {
        var a = tf.tensor1d([1.1, 2.1], 'float32');
        var b = tf.tensor1d([1.1, 2.1, 3.1], 'float32');
        var f = function () {
            tf.less(a, b);
        };
        expect(f).toThrowError();
    });
    it('NaNs in Tensor1D - float32', function () {
        var a = tf.tensor1d([1.1, NaN, 2.1], 'float32');
        var b = tf.tensor1d([2.1, 3.1, NaN], 'float32');
        var res = tf.less(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 0]);
    });
    // Tensor2D:
    it('Tensor2D - int32', function () {
        var a = tf.tensor2d([[1, 4, 5], [8, 9, 12]], [2, 3], 'int32');
        var b = tf.tensor2d([[2, 3, 6], [7, 10, 11]], [2, 3], 'int32');
        var res = tf.less(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 1, 0, 1, 0]);
        a = tf.tensor2d([[0, 0], [1, 1]], [2, 2], 'int32');
        b = tf.tensor2d([[0, 0], [1, 1]], [2, 2], 'int32');
        res = tf.less(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 0, 0, 0]);
    });
    it('Tensor2D - float32', function () {
        var a = tf.tensor2d([[1.1, 4.1, 5.1], [8.1, 9.1, 12.1]], [2, 3], 'float32');
        var b = tf.tensor2d([[2.1, 3.1, 6.1], [7.1, 10.1, 11.1]], [2, 3], 'float32');
        var res = tf.less(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 1, 0, 1, 0]);
        a = tf.tensor2d([[0.2, 0.2], [1.2, 1.2]], [2, 2], 'float32');
        b = tf.tensor2d([[0.2, 0.2], [1.2, 1.2]], [2, 2], 'float32');
        res = tf.less(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 0, 0, 0]);
    });
    it('broadcasting Tensor2D shapes - int32', function () {
        var a = tf.tensor2d([[3], [7]], [2, 1], 'int32');
        var b = tf.tensor2d([[2, 3, 4], [7, 8, 9]], [2, 3], 'int32');
        var res = tf.less(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 0, 1, 0, 1, 1]);
    });
    it('broadcasting Tensor2D shapes - float32', function () {
        var a = tf.tensor2d([[1.1], [7.1]], [2, 1], 'float32');
        var b = tf.tensor2d([[0.1, 1.1, 2.1], [7.1, 8.1, 9.1]], [2, 3], 'float32');
        var res = tf.less(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 0, 1, 0, 1, 1]);
    });
    it('NaNs in Tensor2D - float32', function () {
        var a = tf.tensor2d([[1.1, NaN], [0.1, NaN]], [2, 2], 'float32');
        var b = tf.tensor2d([[0.1, NaN], [1.1, NaN]], [2, 2], 'float32');
        var res = tf.less(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 0, 1, 0]);
    });
    // Tensor3D:
    it('Tensor3D - int32', function () {
        var a = tf.tensor3d([[[1], [4], [5]], [[8], [9], [12]]], [2, 3, 1], 'int32');
        var b = tf.tensor3d([[[2], [3], [6]], [[7], [10], [11]]], [2, 3, 1], 'int32');
        var res = tf.less(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 1, 0, 1, 0]);
        a = tf.tensor3d([[[0], [0], [0]], [[1], [1], [1]]], [2, 3, 1], 'int32');
        b = tf.tensor3d([[[0], [0], [0]], [[1], [1], [1]]], [2, 3, 1], 'int32');
        res = tf.less(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 0, 0, 0, 0, 0]);
    });
    it('Tensor3D - float32', function () {
        var a = tf.tensor3d([[[1.1], [4.1], [5.1]], [[8.1], [9.1], [12.1]]], [2, 3, 1], 'float32');
        var b = tf.tensor3d([[[2.1], [3.1], [6.1]], [[7.1], [10.1], [11.1]]], [2, 3, 1], 'float32');
        var res = tf.less(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 1, 0, 1, 0]);
        a = tf.tensor3d([[[0.1], [0.1], [0.1]], [[1.1], [1.1], [1.0]]], [2, 3, 1], 'float32');
        b = tf.tensor3d([[[0.1], [0.1], [0.1]], [[1.1], [1.1], [1.1]]], [2, 3, 1], 'float32');
        res = tf.less(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 0, 0, 0, 0, 1]);
    });
    it('broadcasting Tensor3D shapes - int32', function () {
        var a = tf.tensor3d([[[1, 0], [2, 3], [4, 5]], [[6, 7], [9, 8], [10, 11]]], [2, 3, 2], 'int32');
        var b = tf.tensor3d([[[1], [2], [3]], [[7], [10], [9]]], [2, 3, 1], 'int32');
        var res = tf.less(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 1, 0, 0, 0, 0, 1, 0, 1, 1, 0, 0]);
    });
    it('broadcasting Tensor3D float32', function () {
        var a = tf.tensor3d([
            [[1.1, 0.1], [2.1, 3.1], [4.1, 5.1]],
            [[6.1, 7.1], [9.1, 8.1], [10.1, 11.1]]
        ], [2, 3, 2], 'float32');
        var b = tf.tensor3d([[[1.1], [2.1], [3.1]], [[7.1], [10.1], [9.1]]], [2, 3, 1], 'float32');
        var res = tf.less(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 1, 0, 0, 0, 0, 1, 0, 1, 1, 0, 0]);
    });
    it('NaNs in Tensor3D - float32', function () {
        var a = tf.tensor3d([[[1.1], [NaN], [1.1]], [[0.1], [0.1], [0.1]]], [2, 3, 1], 'float32');
        var b = tf.tensor3d([[[0.1], [0.1], [1.1]], [[1.1], [0.1], [NaN]]], [2, 3, 1], 'float32');
        var res = tf.less(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 0, 0, 1, 0, 0]);
    });
    // Tensor4D:
    it('Tensor4D - int32', function () {
        var a = tf.tensor4d([1, 4, 5, 8], [2, 2, 1, 1], 'int32');
        var b = tf.tensor4d([2, 3, 6, 7], [2, 2, 1, 1], 'int32');
        var res = tf.less(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 1, 0]);
        a = tf.tensor4d([0, 1, 2, 3], [2, 2, 1, 1], 'int32');
        b = tf.tensor4d([0, 1, 2, 3], [2, 2, 1, 1], 'int32');
        res = tf.less(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 0, 0, 0]);
        a = tf.tensor4d([1, 1, 1, 1], [2, 2, 1, 1], 'int32');
        b = tf.tensor4d([2, 2, 2, 2], [2, 2, 1, 1], 'int32');
        res = tf.less(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 1, 1, 1]);
    });
    it('Tensor4D - float32', function () {
        var a = tf.tensor4d([1.1, 4.1, 5.1, 8.1], [2, 2, 1, 1], 'float32');
        var b = tf.tensor4d([2.1, 3.1, 6.1, 7.1], [2, 2, 1, 1], 'float32');
        var res = tf.less(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 1, 0]);
        a = tf.tensor4d([0.1, 1.1, 2.2, 3.3], [2, 2, 1, 1], 'float32');
        b = tf.tensor4d([0.1, 1.1, 2.2, 3.3], [2, 2, 1, 1], 'float32');
        res = tf.less(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 0, 0, 0]);
        a = tf.tensor4d([0.1, 0.1, 0.1, 0.1], [2, 2, 1, 1], 'float32');
        b = tf.tensor4d([1.1, 1.1, 1.1, 1.1], [2, 2, 1, 1], 'float32');
        res = tf.less(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 1, 1, 1]);
    });
    it('broadcasting Tensor4D shapes - int32', function () {
        var a = tf.tensor4d([1, 2, 5, 9], [2, 2, 1, 1], 'int32');
        var b = tf.tensor4d([[[[1, 2]], [[3, 4]]], [[[5, 6]], [[7, 8]]]], [2, 2, 1, 2], 'int32');
        var res = tf.less(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 1, 1, 1, 0, 1, 0, 0]);
    });
    it('broadcasting Tensor4D shapes - float32', function () {
        var a = tf.tensor4d([1.1, 2.1, 5.1, 9.1], [2, 2, 1, 1], 'float32');
        var b = tf.tensor4d([[[[1.1, 2.1]], [[3.1, 4.1]]], [[[5.1, 6.1]], [[7.1, 8.1]]]], [2, 2, 1, 2], 'float32');
        var res = tf.less(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 1, 1, 1, 0, 1, 0, 0]);
    });
    it('NaNs in Tensor4D - float32', function () {
        var a = tf.tensor4d([1.1, NaN, 0.1, 0.1], [2, 2, 1, 1], 'float32');
        var b = tf.tensor4d([0.1, 1.1, 1.1, NaN], [2, 2, 1, 1], 'float32');
        var res = tf.less(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 0, 1, 0]);
    });
    it('throws when passed a as a non-tensor', function () {
        expect(function () { return tf.less({}, tf.scalar(1)); })
            .toThrowError(/Argument 'a' passed to 'less' must be a Tensor/);
    });
    it('throws when passed b as a non-tensor', function () {
        expect(function () { return tf.less(tf.scalar(1), {}); })
            .toThrowError(/Argument 'b' passed to 'less' must be a Tensor/);
    });
    it('accepts a tensor-like object', function () {
        var a = [1, 4, 5];
        var b = [2, 3, 5];
        var res = tf.less(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 0]);
    });
});
jasmine_util_1.describeWithFlags('lessStrict', test_util_1.ALL_ENVS, function () {
    it('Tensor1D - strict version throws when a and b are different shape', function () {
        var a = tf.tensor1d([2]);
        var b = tf.tensor1d([4, 2, -1]);
        expect(function () { return tf.lessStrict(a, b); }).toThrowError();
        expect(function () { return tf.lessStrict(b, a); }).toThrowError();
    });
    // Tensor2D:
    it('Tensor2D - strict version throws when a and b are different shape', function () {
        var a = tf.tensor2d([[1.1], [7.1]], [2, 1], 'float32');
        var b = tf.tensor2d([[0.1, 1.1, 2.1], [7.1, 8.1, 9.1]], [2, 3], 'float32');
        expect(function () { return tf.lessStrict(a, b); }).toThrowError();
        expect(function () { return tf.lessStrict(b, a); }).toThrowError();
    });
    // Tensor3D:
    it('Tensor3D - strict version throws when a and b are different shape', function () {
        var a = tf.tensor3d([
            [[1.1, 0.1], [2.1, 3.1], [4.1, 5.1]],
            [[6.1, 7.1], [9.1, 8.1], [10.1, 11.1]]
        ], [2, 3, 2], 'float32');
        var b = tf.tensor3d([[[1.1], [2.1], [3.1]], [[7.1], [10.1], [9.1]]], [2, 3, 1], 'float32');
        expect(function () { return tf.lessStrict(a, b); }).toThrowError();
        expect(function () { return tf.lessStrict(b, a); }).toThrowError();
    });
    // Tensor4D:
    it('Tensor4D - strict version throws when a and b are different shape', function () {
        var a = tf.tensor4d([1.1, 2.1, 5.1, 9.1], [2, 2, 1, 1], 'float32');
        var b = tf.tensor4d([[[[1.1, 2.1]], [[3.1, 4.1]]], [[[5.1, 6.1]], [[7.1, 8.1]]]], [2, 2, 1, 2], 'float32');
        expect(function () { return tf.lessStrict(a, b); }).toThrowError();
        expect(function () { return tf.lessStrict(b, a); }).toThrowError();
    });
    it('accepts a tensor-like object', function () {
        var a = [1, 4, 5];
        var b = [2, 3, 5];
        var res = tf.lessStrict(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 0]);
    });
});
jasmine_util_1.describeWithFlags('lessEqual', test_util_1.ALL_ENVS, function () {
    it('Tensor1D - int32', function () {
        var a = tf.tensor1d([1, 4, 5], 'int32');
        var b = tf.tensor1d([2, 3, 5], 'int32');
        var res = tf.lessEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 1]);
        a = tf.tensor1d([2, 2, 2], 'int32');
        b = tf.tensor1d([2, 2, 2], 'int32');
        res = tf.lessEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 1, 1]);
        a = tf.tensor1d([0, 0], 'int32');
        b = tf.tensor1d([3, 3], 'int32');
        res = tf.lessEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 1]);
    });
    it('Tensor1D - float32', function () {
        var a = tf.tensor1d([1.1, 4.1, 5.1], 'float32');
        var b = tf.tensor1d([2.2, 3.2, 5.1], 'float32');
        var res = tf.lessEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 1]);
        a = tf.tensor1d([2.31, 2.31, 2.31], 'float32');
        b = tf.tensor1d([2.31, 2.31, 2.31], 'float32');
        res = tf.lessEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 1, 1]);
        a = tf.tensor1d([0.45, 0.123], 'float32');
        b = tf.tensor1d([3.123, 3.321], 'float32');
        res = tf.lessEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 1]);
    });
    it('upcasts when dtypes dont match', function () {
        var a = [1.1, 4.1, 5];
        var b = [2.2, 3.2, 5];
        var res = tf.lessEqual(tf.tensor(a, [3], 'float32'), tf.tensor(b, [3], 'int32'));
        expect(res.dtype).toBe('bool');
        expect(res.shape).toEqual([3]);
        test_util_1.expectArraysClose(res, [1, 0, 1]);
        res =
            tf.lessEqual(tf.tensor(a, [3], 'int32'), tf.tensor(b, [3], 'bool'));
        expect(res.dtype).toBe('bool');
        expect(res.shape).toEqual([3]);
        test_util_1.expectArraysClose(res, [1, 0, 0]);
    });
    it('TensorLike', function () {
        var a = [1.1, 4.1, 5.1];
        var b = [2.2, 3.2, 5.1];
        var res = tf.lessEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 1]);
    });
    it('TensorLike Chained', function () {
        var a = tf.tensor1d([1.1, 4.1, 5.1], 'float32');
        var b = [2.2, 3.2, 5.1];
        var res = a.lessEqual(b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 1]);
    });
    it('mismatched Tensor1D shapes - int32', function () {
        var a = tf.tensor1d([1, 2], 'int32');
        var b = tf.tensor1d([1, 2, 3], 'int32');
        var f = function () {
            tf.lessEqual(a, b);
        };
        expect(f).toThrowError();
    });
    it('mismatched Tensor1D shapes - float32', function () {
        var a = tf.tensor1d([1.1, 2.1], 'float32');
        var b = tf.tensor1d([1.1, 2.1, 3.1], 'float32');
        var f = function () {
            tf.lessEqual(a, b);
        };
        expect(f).toThrowError();
    });
    it('NaNs in Tensor1D - float32', function () {
        var a = tf.tensor1d([1.1, NaN, 2.1], 'float32');
        var b = tf.tensor1d([2.1, 3.1, NaN], 'float32');
        var res = tf.lessEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 0]);
    });
    // Tensor2D:
    it('Tensor2D - int32', function () {
        var a = tf.tensor2d([[1, 4, 5], [8, 9, 12]], [2, 3], 'int32');
        var b = tf.tensor2d([[2, 3, 6], [7, 10, 11]], [2, 3], 'int32');
        var res = tf.lessEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 1, 0, 1, 0]);
        a = tf.tensor2d([[0, 0], [1, 1]], [2, 2], 'int32');
        b = tf.tensor2d([[0, 0], [1, 1]], [2, 2], 'int32');
        res = tf.lessEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 1, 1, 1]);
    });
    it('Tensor2D - float32', function () {
        var a = tf.tensor2d([[1.1, 4.1, 5.1], [8.1, 9.1, 12.1]], [2, 3], 'float32');
        var b = tf.tensor2d([[2.1, 3.1, 6.1], [7.1, 10.1, 11.1]], [2, 3], 'float32');
        var res = tf.lessEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 1, 0, 1, 0]);
        a = tf.tensor2d([[0.2, 0.2], [1.2, 1.2]], [2, 2], 'float32');
        b = tf.tensor2d([[0.2, 0.2], [1.2, 1.2]], [2, 2], 'float32');
        res = tf.lessEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 1, 1, 1]);
    });
    it('broadcasting Tensor2D shapes - int32', function () {
        var a = tf.tensor2d([[3], [7]], [2, 1], 'int32');
        var b = tf.tensor2d([[2, 3, 4], [7, 8, 9]], [2, 3], 'int32');
        var res = tf.lessEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 1, 1, 1, 1, 1]);
    });
    it('broadcasting Tensor2D shapes - float32', function () {
        var a = tf.tensor2d([[1.1], [7.1]], [2, 1], 'float32');
        var b = tf.tensor2d([[0.1, 1.1, 2.1], [7.1, 8.1, 9.1]], [2, 3], 'float32');
        var res = tf.lessEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 1, 1, 1, 1, 1]);
    });
    it('NaNs in Tensor2D - float32', function () {
        var a = tf.tensor2d([[1.1, NaN], [0.1, NaN]], [2, 2], 'float32');
        var b = tf.tensor2d([[0.1, NaN], [1.1, NaN]], [2, 2], 'float32');
        var res = tf.lessEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 0, 1, 0]);
    });
    // Tensor3D:
    it('Tensor3D - int32', function () {
        var a = tf.tensor3d([[[1], [4], [5]], [[8], [9], [12]]], [2, 3, 1], 'int32');
        var b = tf.tensor3d([[[2], [3], [6]], [[7], [10], [11]]], [2, 3, 1], 'int32');
        var res = tf.lessEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 1, 0, 1, 0]);
        a = tf.tensor3d([[[0], [0], [0]], [[1], [1], [1]]], [2, 3, 1], 'int32');
        b = tf.tensor3d([[[0], [0], [0]], [[1], [1], [1]]], [2, 3, 1], 'int32');
        res = tf.lessEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 1, 1, 1, 1, 1]);
    });
    it('Tensor3D - float32', function () {
        var a = tf.tensor3d([[[1.1], [4.1], [5.1]], [[8.1], [9.1], [12.1]]], [2, 3, 1], 'float32');
        var b = tf.tensor3d([[[2.1], [3.1], [6.1]], [[7.1], [10.1], [11.1]]], [2, 3, 1], 'float32');
        var res = tf.lessEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 1, 0, 1, 0]);
        a = tf.tensor3d([[[0.1], [0.1], [0.1]], [[1.1], [1.1], [1.2]]], [2, 3, 1], 'float32');
        b = tf.tensor3d([[[0.1], [0.1], [0.1]], [[1.1], [1.1], [1.1]]], [2, 3, 1], 'float32');
        res = tf.lessEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 1, 1, 1, 1, 0]);
    });
    it('broadcasting Tensor3D shapes - int32', function () {
        var a = tf.tensor3d([[[1, 0], [2, 3], [4, 5]], [[6, 7], [9, 8], [10, 11]]], [2, 3, 2], 'int32');
        var b = tf.tensor3d([[[1], [2], [3]], [[7], [10], [9]]], [2, 3, 1], 'int32');
        var res = tf.lessEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 0, 0]);
    });
    it('broadcasting Tensor3D float32', function () {
        var a = tf.tensor3d([
            [[1.1, 0.1], [2.1, 3.1], [4.1, 5.1]],
            [[6.1, 7.1], [9.1, 8.1], [10.1, 11.1]]
        ], [2, 3, 2], 'float32');
        var b = tf.tensor3d([[[1.1], [2.1], [3.1]], [[7.1], [10.1], [9.1]]], [2, 3, 1], 'float32');
        var res = tf.lessEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 1, 1, 0, 0, 0, 1, 1, 1, 1, 0, 0]);
    });
    it('NaNs in Tensor3D - float32', function () {
        var a = tf.tensor3d([[[1.1], [NaN], [1.1]], [[0.1], [0.1], [0.1]]], [2, 3, 1], 'float32');
        var b = tf.tensor3d([[[0.1], [0.1], [1.1]], [[1.1], [0.1], [NaN]]], [2, 3, 1], 'float32');
        var res = tf.lessEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 0, 1, 1, 1, 0]);
    });
    // Tensor4D:
    it('Tensor4D - int32', function () {
        var a = tf.tensor4d([1, 4, 5, 8], [2, 2, 1, 1], 'int32');
        var b = tf.tensor4d([2, 3, 6, 7], [2, 2, 1, 1], 'int32');
        var res = tf.lessEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 1, 0]);
        a = tf.tensor4d([0, 1, 2, 3], [2, 2, 1, 1], 'int32');
        b = tf.tensor4d([0, 1, 2, 3], [2, 2, 1, 1], 'int32');
        res = tf.lessEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 1, 1, 1]);
        a = tf.tensor4d([1, 1, 1, 1], [2, 2, 1, 1], 'int32');
        b = tf.tensor4d([2, 2, 2, 2], [2, 2, 1, 1], 'int32');
        res = tf.lessEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 1, 1, 1]);
    });
    it('Tensor4D - float32', function () {
        var a = tf.tensor4d([1.1, 4.1, 5.1, 8.1], [2, 2, 1, 1], 'float32');
        var b = tf.tensor4d([2.1, 3.1, 6.1, 7.1], [2, 2, 1, 1], 'float32');
        var res = tf.lessEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 1, 0]);
        a = tf.tensor4d([0.1, 1.1, 2.2, 3.3], [2, 2, 1, 1], 'float32');
        b = tf.tensor4d([0.1, 1.1, 2.2, 3.3], [2, 2, 1, 1], 'float32');
        res = tf.lessEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 1, 1, 1]);
        a = tf.tensor4d([0.1, 0.1, 0.1, 0.1], [2, 2, 1, 1], 'float32');
        b = tf.tensor4d([1.1, 1.1, 1.1, 1.1], [2, 2, 1, 1], 'float32');
        res = tf.lessEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 1, 1, 1]);
    });
    it('broadcasting Tensor4D shapes - int32', function () {
        var a = tf.tensor4d([1, 2, 5, 9], [2, 2, 1, 1], 'int32');
        var b = tf.tensor4d([[[[1, 2]], [[3, 4]]], [[[5, 6]], [[7, 8]]]], [2, 2, 1, 2], 'int32');
        var res = tf.lessEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 1, 1, 1, 1, 1, 0, 0]);
    });
    it('broadcasting Tensor4D shapes - float32', function () {
        var a = tf.tensor4d([1.1, 2.1, 5.1, 9.1], [2, 2, 1, 1], 'float32');
        var b = tf.tensor4d([[[[1.1, 2.1]], [[3.1, 4.1]]], [[[5.1, 6.1]], [[7.1, 8.1]]]], [2, 2, 1, 2], 'float32');
        var res = tf.lessEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 1, 1, 1, 1, 1, 0, 0]);
    });
    it('NaNs in Tensor4D - float32', function () {
        var a = tf.tensor4d([1.1, NaN, 0.1, 0.1], [2, 2, 1, 1], 'float32');
        var b = tf.tensor4d([0.1, 1.1, 1.1, NaN], [2, 2, 1, 1], 'float32');
        var res = tf.lessEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 0, 1, 0]);
    });
    it('throws when passed a as a non-tensor', function () {
        expect(function () { return tf.lessEqual({}, tf.scalar(1)); })
            .toThrowError(/Argument 'a' passed to 'lessEqual' must be a Tensor/);
    });
    it('throws when passed b as a non-tensor', function () {
        expect(function () { return tf.lessEqual(tf.scalar(1), {}); })
            .toThrowError(/Argument 'b' passed to 'lessEqual' must be a Tensor/);
    });
    it('accepts a tensor-like object', function () {
        var a = [1, 4, 5];
        var b = [2, 3, 5];
        var res = tf.lessEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 1]);
    });
});
jasmine_util_1.describeWithFlags('lessEqualStrict', test_util_1.ALL_ENVS, function () {
    it('Tensor1D - strict version throws when a and b are different shape', function () {
        var a = tf.tensor1d([2]);
        var b = tf.tensor1d([4, 2, -1]);
        expect(function () { return tf.lessEqualStrict(a, b); }).toThrowError();
        expect(function () { return tf.lessEqualStrict(b, a); }).toThrowError();
    });
    // Tensor2D:
    it('Tensor2D - strict version throws when a and b are different shape', function () {
        var a = tf.tensor2d([[1.1], [7.1]], [2, 1], 'float32');
        var b = tf.tensor2d([[0.1, 1.1, 2.1], [7.1, 8.1, 9.1]], [2, 3], 'float32');
        expect(function () { return tf.lessEqualStrict(a, b); }).toThrowError();
        expect(function () { return tf.lessEqualStrict(b, a); }).toThrowError();
    });
    // Tensor3D:
    it('Tensor3D - strict version throws when a and b are different shape', function () {
        var a = tf.tensor3d([
            [[1.1, 0.1], [2.1, 3.1], [4.1, 5.1]],
            [[6.1, 7.1], [9.1, 8.1], [10.1, 11.1]]
        ], [2, 3, 2], 'float32');
        var b = tf.tensor3d([[[1.1], [2.1], [3.1]], [[7.1], [10.1], [9.1]]], [2, 3, 1], 'float32');
        expect(function () { return tf.lessEqualStrict(a, b); }).toThrowError();
        expect(function () { return tf.lessEqualStrict(b, a); }).toThrowError();
    });
    // Tensor4D:
    it('Tensor4D - strict version throws when a and b are different shape', function () {
        var a = tf.tensor4d([1.1, 2.1, 5.1, 9.1], [2, 2, 1, 1], 'float32');
        var b = tf.tensor4d([[[[1.1, 2.1]], [[3.1, 4.1]]], [[[5.1, 6.1]], [[7.1, 8.1]]]], [2, 2, 1, 2], 'float32');
        expect(function () { return tf.lessEqualStrict(a, b); }).toThrowError();
        expect(function () { return tf.lessEqualStrict(b, a); }).toThrowError();
    });
    it('accepts a tensor-like object', function () {
        var a = [1, 4, 5];
        var b = [2, 3, 5];
        var res = tf.lessEqualStrict(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 1]);
    });
});
jasmine_util_1.describeWithFlags('greater', test_util_1.ALL_ENVS, function () {
    it('Tensor1D - int32', function () {
        var a = tf.tensor1d([1, 4, 5], 'int32');
        var b = tf.tensor1d([2, 3, 5], 'int32');
        var res = tf.greater(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 1, 0]);
        a = tf.tensor1d([2, 2, 2], 'int32');
        b = tf.tensor1d([2, 2, 2], 'int32');
        res = tf.greater(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 0, 0]);
        a = tf.tensor1d([3, 3], 'int32');
        b = tf.tensor1d([0, 0], 'int32');
        res = tf.greater(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 1]);
    });
    it('Tensor1D - float32', function () {
        var a = tf.tensor1d([1.1, 4.1, 5.1], 'float32');
        var b = tf.tensor1d([2.2, 3.2, 5.1], 'float32');
        var res = tf.greater(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 1, 0]);
        a = tf.tensor1d([2.31, 2.31, 2.31], 'float32');
        b = tf.tensor1d([2.31, 2.31, 2.31], 'float32');
        res = tf.greater(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 0, 0]);
        a = tf.tensor1d([3.123, 3.321], 'float32');
        b = tf.tensor1d([0.45, 0.123], 'float32');
        res = tf.greater(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 1]);
    });
    it('upcasts when dtypes dont match', function () {
        var a = [1.1, 4.1, 5.2];
        var b = [2.2, 3.2, 5.1];
        var res = tf.greater(tf.tensor(a, [3], 'float32'), tf.tensor(b, [3], 'int32'));
        expect(res.dtype).toBe('bool');
        expect(res.shape).toEqual([3]);
        test_util_1.expectArraysClose(res, [0, 1, 1]);
        res =
            tf.greater(tf.tensor(a, [3], 'int32'), tf.tensor(b, [3], 'bool'));
        expect(res.dtype).toBe('bool');
        expect(res.shape).toEqual([3]);
        test_util_1.expectArraysClose(res, [0, 1, 1]);
    });
    it('TensorLike', function () {
        var a = [1.1, 4.1, 5.1];
        var b = [2.2, 3.2, 5.1];
        var res = tf.greater(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 1, 0]);
    });
    it('TensorLike Chained', function () {
        var a = tf.tensor1d([1.1, 4.1, 5.1], 'float32');
        var b = [2.2, 3.2, 5.1];
        var res = a.greater(b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 1, 0]);
    });
    it('mismatched Tensor1D shapes - int32', function () {
        var a = tf.tensor1d([1, 2], 'int32');
        var b = tf.tensor1d([1, 2, 3], 'int32');
        var f = function () {
            tf.greater(a, b);
        };
        expect(f).toThrowError();
    });
    it('mismatched Tensor1D shapes - float32', function () {
        var a = tf.tensor1d([1.1, 2.1], 'float32');
        var b = tf.tensor1d([1.1, 2.1, 3.1], 'float32');
        var f = function () {
            tf.greater(a, b);
        };
        expect(f).toThrowError();
    });
    it('NaNs in Tensor1D - float32', function () {
        var a = tf.tensor1d([1.1, NaN, 2.1], 'float32');
        var b = tf.tensor1d([2.1, 3.1, NaN], 'float32');
        var res = tf.greater(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 0, 0]);
    });
    // Tensor2D:
    it('Tensor2D - int32', function () {
        var a = tf.tensor2d([[1, 4, 5], [8, 9, 11]], [2, 3], 'int32');
        var b = tf.tensor2d([[2, 3, 6], [7, 10, 11]], [2, 3], 'int32');
        var res = tf.greater(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 1, 0, 1, 0, 0]);
        a = tf.tensor2d([[0, 0], [1, 1]], [2, 2], 'int32');
        b = tf.tensor2d([[0, 0], [1, 1]], [2, 2], 'int32');
        res = tf.greater(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 0, 0, 0]);
    });
    it('Tensor2D - float32', function () {
        var a = tf.tensor2d([[1.1, 4.1, 5.1], [8.1, 9.1, 11.1]], [2, 3], 'float32');
        var b = tf.tensor2d([[2.1, 3.1, 6.1], [7.1, 10.1, 11.1]], [2, 3], 'float32');
        var res = tf.greater(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 1, 0, 1, 0, 0]);
        a = tf.tensor2d([[0.2, 0.2], [1.2, 1.2]], [2, 2], 'float32');
        b = tf.tensor2d([[0.2, 0.2], [1.2, 1.2]], [2, 2], 'float32');
        res = tf.greater(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 0, 0, 0]);
    });
    it('broadcasting Tensor2D shapes - int32', function () {
        var a = tf.tensor2d([[3], [7]], [2, 1], 'int32');
        var b = tf.tensor2d([[2, 3, 4], [7, 8, 9]], [2, 3], 'int32');
        var res = tf.greater(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 0, 0, 0, 0]);
    });
    it('broadcasting Tensor2D shapes - float32', function () {
        var a = tf.tensor2d([[1.1], [7.1]], [2, 1], 'float32');
        var b = tf.tensor2d([[0.1, 1.1, 2.1], [7.1, 8.1, 9.1]], [2, 3], 'float32');
        var res = tf.greater(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 0, 0, 0, 0]);
    });
    it('NaNs in Tensor2D - float32', function () {
        var a = tf.tensor2d([[1.1, NaN], [0.1, NaN]], [2, 2], 'float32');
        var b = tf.tensor2d([[0.1, NaN], [1.1, NaN]], [2, 2], 'float32');
        var res = tf.greater(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 0, 0]);
    });
    // Tensor3D:
    it('Tensor3D - int32', function () {
        var a = tf.tensor3d([[[1], [4], [5]], [[8], [9], [11]]], [2, 3, 1], 'int32');
        var b = tf.tensor3d([[[2], [3], [6]], [[7], [10], [11]]], [2, 3, 1], 'int32');
        var res = tf.greater(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 1, 0, 1, 0, 0]);
        a = tf.tensor3d([[[0], [0], [0]], [[1], [1], [1]]], [2, 3, 1], 'int32');
        b = tf.tensor3d([[[0], [0], [0]], [[1], [1], [1]]], [2, 3, 1], 'int32');
        res = tf.greater(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 0, 0, 0, 0, 0]);
    });
    it('Tensor3D - float32', function () {
        var a = tf.tensor3d([[[1.1], [4.1], [5.1]], [[8.1], [9.1], [11.1]]], [2, 3, 1], 'float32');
        var b = tf.tensor3d([[[2.1], [3.1], [6.1]], [[7.1], [10.1], [11.1]]], [2, 3, 1], 'float32');
        var res = tf.greater(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 1, 0, 1, 0, 0]);
        a = tf.tensor3d([[[0.1], [0.1], [0.1]], [[1.1], [1.1], [1.2]]], [2, 3, 1], 'float32');
        b = tf.tensor3d([[[0.1], [0.1], [0.1]], [[1.1], [1.1], [1.1]]], [2, 3, 1], 'float32');
        res = tf.greater(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 0, 0, 0, 0, 1]);
    });
    it('broadcasting Tensor3D shapes - int32', function () {
        var a = tf.tensor3d([[[1, 0], [2, 3], [4, 5]], [[6, 7], [9, 8], [10, 11]]], [2, 3, 2], 'int32');
        var b = tf.tensor3d([[[1], [2], [3]], [[7], [10], [9]]], [2, 3, 1], 'int32');
        var res = tf.greater(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1]);
    });
    it('broadcasting Tensor3D float32', function () {
        var a = tf.tensor3d([
            [[1.1, 0.1], [2.1, 3.1], [4.1, 5.1]],
            [[6.1, 7.1], [9.1, 8.1], [10.1, 11.1]]
        ], [2, 3, 2], 'float32');
        var b = tf.tensor3d([[[1.1], [2.1], [3.1]], [[7.1], [10.1], [9.1]]], [2, 3, 1], 'float32');
        var res = tf.greater(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 1, 1]);
    });
    it('NaNs in Tensor3D - float32', function () {
        var a = tf.tensor3d([[[1.1], [NaN], [1.1]], [[0.1], [0.1], [0.1]]], [2, 3, 1], 'float32');
        var b = tf.tensor3d([[[0.1], [0.1], [1.1]], [[1.1], [0.1], [NaN]]], [2, 3, 1], 'float32');
        var res = tf.greater(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 0, 0, 0, 0]);
    });
    // Tensor4D:
    it('Tensor4D - int32', function () {
        var a = tf.tensor4d([1, 4, 5, 8], [2, 2, 1, 1], 'int32');
        var b = tf.tensor4d([2, 3, 6, 8], [2, 2, 1, 1], 'int32');
        var res = tf.greater(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 1, 0, 0]);
        a = tf.tensor4d([0, 1, 2, 3], [2, 2, 1, 1], 'int32');
        b = tf.tensor4d([0, 1, 2, 3], [2, 2, 1, 1], 'int32');
        res = tf.greater(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 0, 0, 0]);
        a = tf.tensor4d([2, 2, 2, 2], [2, 2, 1, 1], 'int32');
        b = tf.tensor4d([1, 1, 1, 1], [2, 2, 1, 1], 'int32');
        res = tf.greater(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 1, 1, 1]);
    });
    it('Tensor4D - float32', function () {
        var a = tf.tensor4d([1.1, 4.1, 5.1, 8.1], [2, 2, 1, 1], 'float32');
        var b = tf.tensor4d([2.1, 3.1, 6.1, 8.1], [2, 2, 1, 1], 'float32');
        var res = tf.greater(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 1, 0, 0]);
        a = tf.tensor4d([0.1, 1.1, 2.2, 3.3], [2, 2, 1, 1], 'float32');
        b = tf.tensor4d([0.1, 1.1, 2.2, 3.3], [2, 2, 1, 1], 'float32');
        res = tf.greater(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 0, 0, 0]);
        a = tf.tensor4d([1.1, 1.1, 1.1, 1.1], [2, 2, 1, 1], 'float32');
        b = tf.tensor4d([0.1, 0.1, 0.1, 0.1], [2, 2, 1, 1], 'float32');
        res = tf.greater(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 1, 1, 1]);
    });
    it('broadcasting Tensor4D shapes - int32', function () {
        var a = tf.tensor4d([1, 2, 5, 9], [2, 2, 1, 1], 'int32');
        var b = tf.tensor4d([[[[1, 2]], [[3, 4]]], [[[5, 6]], [[7, 8]]]], [2, 2, 1, 2], 'int32');
        var res = tf.greater(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 0, 0, 0, 0, 0, 1, 1]);
    });
    it('broadcasting Tensor4D shapes - float32', function () {
        var a = tf.tensor4d([1.1, 2.1, 5.1, 9.1], [2, 2, 1, 1], 'float32');
        var b = tf.tensor4d([[[[1.1, 2.1]], [[3.1, 4.1]]], [[[5.1, 6.1]], [[7.1, 8.1]]]], [2, 2, 1, 2], 'float32');
        var res = tf.greater(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 0, 0, 0, 0, 0, 1, 1]);
    });
    it('NaNs in Tensor4D - float32', function () {
        var a = tf.tensor4d([1.1, NaN, 0.1, 0.1], [2, 2, 1, 1], 'float32');
        var b = tf.tensor4d([0.1, 1.1, 1.1, NaN], [2, 2, 1, 1], 'float32');
        var res = tf.greater(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 0, 0]);
    });
    it('throws when passed a as a non-tensor', function () {
        expect(function () { return tf.greater({}, tf.scalar(1)); })
            .toThrowError(/Argument 'a' passed to 'greater' must be a Tensor/);
    });
    it('throws when passed b as a non-tensor', function () {
        expect(function () { return tf.greater(tf.scalar(1), {}); })
            .toThrowError(/Argument 'b' passed to 'greater' must be a Tensor/);
    });
    it('accepts a tensor-like object', function () {
        var a = [1, 4, 5];
        var b = [2, 3, 5];
        var res = tf.greater(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 1, 0]);
    });
    it('works with 0 sized tensors', function () {
        var a = tf.tensor2d([], [0, 5]);
        var b = tf.tensor1d([1, 2, 3, 4, 5]);
        var res = tf.greater(a, b);
        expect(res.dtype).toBe('bool');
        expect(res.shape).toEqual([0, 5]);
        test_util_1.expectArraysClose(res, []);
    });
});
jasmine_util_1.describeWithFlags('greaterStrict', test_util_1.ALL_ENVS, function () {
    it('Tensor1D - strict version throws when a and b are different shape', function () {
        var a = tf.tensor1d([2]);
        var b = tf.tensor1d([4, 2, -1]);
        expect(function () { return tf.greaterStrict(a, b); }).toThrowError();
        expect(function () { return tf.greaterStrict(b, a); }).toThrowError();
    });
    // Tensor2D:
    it('Tensor2D - strict version throws when a and b are different shape', function () {
        var a = tf.tensor2d([[1.1], [7.1]], [2, 1], 'float32');
        var b = tf.tensor2d([[0.1, 1.1, 2.1], [7.1, 8.1, 9.1]], [2, 3], 'float32');
        expect(function () { return tf.greaterStrict(a, b); }).toThrowError();
        expect(function () { return tf.greaterStrict(b, a); }).toThrowError();
    });
    // Tensor3D:
    it('Tensor3D - strict version throws when a and b are different shape', function () {
        var a = tf.tensor3d([
            [[1.1, 0.1], [2.1, 3.1], [4.1, 5.1]],
            [[6.1, 7.1], [9.1, 8.1], [10.1, 11.1]]
        ], [2, 3, 2], 'float32');
        var b = tf.tensor3d([[[1.1], [2.1], [3.1]], [[7.1], [10.1], [9.1]]], [2, 3, 1], 'float32');
        expect(function () { return tf.greaterStrict(a, b); }).toThrowError();
        expect(function () { return tf.greaterStrict(b, a); }).toThrowError();
    });
    // Tensor4D:
    it('Tensor4D - strict version throws when a and b are different shape', function () {
        var a = tf.tensor4d([1.1, 2.1, 5.1, 9.1], [2, 2, 1, 1], 'float32');
        var b = tf.tensor4d([[[[1.1, 2.1]], [[3.1, 4.1]]], [[[5.1, 6.1]], [[7.1, 8.1]]]], [2, 2, 1, 2], 'float32');
        expect(function () { return tf.greaterStrict(a, b); }).toThrowError();
        expect(function () { return tf.greaterStrict(b, a); }).toThrowError();
    });
    it('accepts a tensor-like object', function () {
        var a = [1, 4, 5];
        var b = [2, 3, 5];
        var res = tf.greaterStrict(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 1, 0]);
    });
});
jasmine_util_1.describeWithFlags('greaterEqual', test_util_1.ALL_ENVS, function () {
    // Tensor1D:
    it('Tensor1D - int32', function () {
        var a = tf.tensor1d([1, 4, 5], 'int32');
        var b = tf.tensor1d([2, 3, 5], 'int32');
        var res = tf.greaterEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 1, 1]);
        a = tf.tensor1d([2, 2, 2], 'int32');
        b = tf.tensor1d([2, 2, 2], 'int32');
        res = tf.greaterEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 1, 1]);
        a = tf.tensor1d([0, 0], 'int32');
        b = tf.tensor1d([3, 3], 'int32');
        res = tf.greaterEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 0]);
    });
    it('Tensor1D - float32', function () {
        var a = tf.tensor1d([1.1, 4.1, 5.1], 'float32');
        var b = tf.tensor1d([2.2, 3.2, 5.1], 'float32');
        var res = tf.greaterEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 1, 1]);
        a = tf.tensor1d([2.31, 2.31, 2.31], 'float32');
        b = tf.tensor1d([2.31, 2.31, 2.31], 'float32');
        res = tf.greaterEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 1, 1]);
        a = tf.tensor1d([0.45, 0.123], 'float32');
        b = tf.tensor1d([3.123, 3.321], 'float32');
        res = tf.greaterEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 0]);
    });
    it('upcasts when dtypes dont match', function () {
        var a = [1.1, 4.1, 5];
        var b = [2.2, 3.2, 5];
        var res = tf.greaterEqual(tf.tensor(a, [3], 'float32'), tf.tensor(b, [3], 'int32'));
        expect(res.dtype).toBe('bool');
        expect(res.shape).toEqual([3]);
        test_util_1.expectArraysClose(res, [0, 1, 1]);
        res =
            tf.greaterEqual(tf.tensor(a, [3], 'int32'), tf.tensor(b, [3], 'bool'));
        expect(res.dtype).toBe('bool');
        expect(res.shape).toEqual([3]);
        test_util_1.expectArraysClose(res, [1, 1, 1]);
    });
    it('mismatched Tensor1D shapes - int32', function () {
        var a = tf.tensor1d([1, 2], 'int32');
        var b = tf.tensor1d([1, 2, 3], 'int32');
        var f = function () {
            tf.greaterEqual(a, b);
        };
        expect(f).toThrowError();
    });
    it('mismatched Tensor1D shapes - float32', function () {
        var a = tf.tensor1d([1.1, 2.1], 'float32');
        var b = tf.tensor1d([1.1, 2.1, 3.1], 'float32');
        var f = function () {
            tf.greaterEqual(a, b);
        };
        expect(f).toThrowError();
    });
    it('NaNs in Tensor1D - float32', function () {
        var a = tf.tensor1d([1.1, NaN, 2.1], 'float32');
        var b = tf.tensor1d([2.1, 3.1, NaN], 'float32');
        var res = tf.greaterEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 0, 0]);
    });
    // Tensor2D:
    it('Tensor2D - int32', function () {
        var a = tf.tensor2d([[1, 4, 5], [8, 9, 12]], [2, 3], 'int32');
        var b = tf.tensor2d([[2, 3, 6], [7, 10, 11]], [2, 3], 'int32');
        var res = tf.greaterEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 1, 0, 1, 0, 1]);
        a = tf.tensor2d([[0, 0], [1, 1]], [2, 2], 'int32');
        b = tf.tensor2d([[0, 0], [1, 1]], [2, 2], 'int32');
        res = tf.greaterEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 1, 1, 1]);
    });
    it('Tensor2D - float32', function () {
        var a = tf.tensor2d([[1.1, 4.1, 5.1], [8.1, 9.1, 12.1]], [2, 3], 'float32');
        var b = tf.tensor2d([[2.1, 3.1, 6.1], [7.1, 10.1, 11.1]], [2, 3], 'float32');
        var res = tf.greaterEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 1, 0, 1, 0, 1]);
        a = tf.tensor2d([[0.2, 0.2], [1.2, 1.2]], [2, 2], 'float32');
        b = tf.tensor2d([[0.2, 0.2], [1.2, 1.2]], [2, 2], 'float32');
        res = tf.greaterEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 1, 1, 1]);
    });
    it('broadcasting Tensor2D shapes - int32', function () {
        var a = tf.tensor2d([[3], [7]], [2, 1], 'int32');
        var b = tf.tensor2d([[2, 3, 4], [7, 8, 9]], [2, 3], 'int32');
        var res = tf.greaterEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 1, 0, 1, 0, 0]);
    });
    it('broadcasting Tensor2D shapes - float32', function () {
        var a = tf.tensor2d([[1.1], [7.1]], [2, 1], 'float32');
        var b = tf.tensor2d([[0.1, 1.1, 2.1], [7.1, 8.1, 9.1]], [2, 3], 'float32');
        var res = tf.greaterEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 1, 0, 1, 0, 0]);
    });
    it('NaNs in Tensor2D - float32', function () {
        var a = tf.tensor2d([[1.1, NaN], [0.1, NaN]], [2, 2], 'float32');
        var b = tf.tensor2d([[0.1, NaN], [1.1, NaN]], [2, 2], 'float32');
        var res = tf.greaterEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 0, 0]);
    });
    // Tensor3D:
    it('Tensor3D - int32', function () {
        var a = tf.tensor3d([[[1], [4], [5]], [[8], [9], [12]]], [2, 3, 1], 'int32');
        var b = tf.tensor3d([[[2], [3], [6]], [[7], [10], [11]]], [2, 3, 1], 'int32');
        var res = tf.greaterEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 1, 0, 1, 0, 1]);
        a = tf.tensor3d([[[0], [0], [0]], [[1], [1], [1]]], [2, 3, 1], 'int32');
        b = tf.tensor3d([[[0], [0], [0]], [[1], [1], [1]]], [2, 3, 1], 'int32');
        res = tf.greaterEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 1, 1, 1, 1, 1]);
    });
    it('Tensor3D - float32', function () {
        var a = tf.tensor3d([[[1.1], [4.1], [5.1]], [[8.1], [9.1], [12.1]]], [2, 3, 1], 'float32');
        var b = tf.tensor3d([[[2.1], [3.1], [6.1]], [[7.1], [10.1], [11.1]]], [2, 3, 1], 'float32');
        var res = tf.greaterEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 1, 0, 1, 0, 1]);
        a = tf.tensor3d([[[0.1], [0.1], [0.1]], [[1.1], [1.1], [1.2]]], [2, 3, 1], 'float32');
        b = tf.tensor3d([[[0.1], [0.1], [0.1]], [[1.1], [1.1], [1.1]]], [2, 3, 1], 'float32');
        res = tf.greaterEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 1, 1, 1, 1, 1]);
    });
    it('broadcasting Tensor3D shapes - int32', function () {
        var a = tf.tensor3d([[[1, 0], [2, 3], [4, 5]], [[6, 7], [9, 8], [10, 11]]], [2, 3, 2], 'int32');
        var b = tf.tensor3d([[[1], [2], [3]], [[7], [10], [9]]], [2, 3, 1], 'int32');
        var res = tf.greaterEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 1, 1, 1, 1, 0, 1, 0, 0, 1, 1]);
    });
    it('broadcasting Tensor3D float32', function () {
        var a = tf.tensor3d([
            [[1.1, 0.1], [2.1, 3.1], [4.1, 5.1]],
            [[6.1, 7.1], [9.1, 8.1], [10.1, 11.1]]
        ], [2, 3, 2], 'float32');
        var b = tf.tensor3d([[[1.1], [2.1], [3.1]], [[7.1], [10.1], [9.1]]], [2, 3, 1], 'float32');
        var res = tf.greaterEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 1, 1, 1, 1, 0, 1, 0, 0, 1, 1]);
    });
    it('NaNs in Tensor3D - float32', function () {
        var a = tf.tensor3d([[[1.1], [NaN], [1.1]], [[0.1], [0.1], [0.1]]], [2, 3, 1], 'float32');
        var b = tf.tensor3d([[[0.1], [0.1], [1.1]], [[1.1], [0.1], [NaN]]], [2, 3, 1], 'float32');
        var res = tf.greaterEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 1, 0, 1, 0]);
    });
    // Tensor4D:
    it('Tensor4D - int32', function () {
        var a = tf.tensor4d([1, 4, 5, 8], [2, 2, 1, 1], 'int32');
        var b = tf.tensor4d([2, 3, 6, 7], [2, 2, 1, 1], 'int32');
        var res = tf.greaterEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 1, 0, 1]);
        a = tf.tensor4d([0, 1, 2, 3], [2, 2, 1, 1], 'int32');
        b = tf.tensor4d([0, 1, 2, 3], [2, 2, 1, 1], 'int32');
        res = tf.greaterEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 1, 1, 1]);
        a = tf.tensor4d([1, 1, 1, 1], [2, 2, 1, 1], 'int32');
        b = tf.tensor4d([2, 2, 2, 2], [2, 2, 1, 1], 'int32');
        res = tf.greaterEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 0, 0, 0]);
    });
    it('Tensor4D - float32', function () {
        var a = tf.tensor4d([1.1, 4.1, 5.1, 8.1], [2, 2, 1, 1], 'float32');
        var b = tf.tensor4d([2.1, 3.1, 6.1, 7.1], [2, 2, 1, 1], 'float32');
        var res = tf.greaterEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 1, 0, 1]);
        a = tf.tensor4d([0.1, 1.1, 2.2, 3.3], [2, 2, 1, 1], 'float32');
        b = tf.tensor4d([0.1, 1.1, 2.2, 3.3], [2, 2, 1, 1], 'float32');
        res = tf.greaterEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 1, 1, 1]);
        a = tf.tensor4d([0.1, 0.1, 0.1, 0.1], [2, 2, 1, 1], 'float32');
        b = tf.tensor4d([1.1, 1.1, 1.1, 1.1], [2, 2, 1, 1], 'float32');
        res = tf.greaterEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 0, 0, 0]);
    });
    it('broadcasting Tensor4D shapes - int32', function () {
        var a = tf.tensor4d([1, 2, 5, 9], [2, 2, 1, 1], 'int32');
        var b = tf.tensor4d([[[[1, 2]], [[3, 4]]], [[[5, 6]], [[7, 8]]]], [2, 2, 1, 2], 'int32');
        var res = tf.greaterEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 0, 0, 1, 0, 1, 1]);
    });
    it('broadcasting Tensor4D shapes - float32', function () {
        var a = tf.tensor4d([1.1, 2.1, 5.1, 9.1], [2, 2, 1, 1], 'float32');
        var b = tf.tensor4d([[[[1.1, 2.1]], [[3.1, 4.1]]], [[[5.1, 6.1]], [[7.1, 8.1]]]], [2, 2, 1, 2], 'float32');
        var res = tf.greaterEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 0, 0, 1, 0, 1, 1]);
    });
    it('NaNs in Tensor4D - float32', function () {
        var a = tf.tensor4d([1.1, NaN, 0.1, 0.1], [2, 2, 1, 1], 'float32');
        var b = tf.tensor4d([0.1, 1.1, 1.1, NaN], [2, 2, 1, 1], 'float32');
        var res = tf.greaterEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [1, 0, 0, 0]);
    });
    it('throws when passed a as a non-tensor', function () {
        expect(function () { return tf.greaterEqual({}, tf.scalar(1)); })
            .toThrowError(/Argument 'a' passed to 'greaterEqual' must be a Tensor/);
    });
    it('throws when passed b as a non-tensor', function () {
        expect(function () { return tf.greaterEqual(tf.scalar(1), {}); })
            .toThrowError(/Argument 'b' passed to 'greaterEqual' must be a Tensor/);
    });
    it('accepts a tensor-like object', function () {
        var a = [1, 4, 5];
        var b = [2, 3, 5];
        var res = tf.greaterEqual(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 1, 1]);
    });
    it('has gradient', function () {
        var a = tf.tensor1d([3, 2, 5]);
        var b = tf.tensor1d([4, 1, 5]);
        var dy = tf.ones([3], 'float32');
        var da = tf.grad(function (a) { return tf.greaterEqual(a, b); })(a, dy);
        expect(da.dtype).toBe('float32');
        expect(da.shape).toEqual([3]);
        test_util_1.expectArraysClose(da, [0, 0, 0]);
    });
    it('gradient with clones', function () {
        var a = tf.tensor1d([3, 2, 5]);
        var b = tf.tensor1d([4, 1, 5]);
        var dy = tf.ones([3], 'float32');
        var da = tf.grad(function (a) {
            return tf.greaterEqual(a.clone(), b.clone()).clone();
        })(a, dy);
        expect(da.dtype).toBe('float32');
        expect(da.shape).toEqual([3]);
        test_util_1.expectArraysClose(da, [0, 0, 0]);
    });
});
jasmine_util_1.describeWithFlags('greaterEqualStrict', test_util_1.ALL_ENVS, function () {
    it('Tensor1D - strict version throws when a and b are different shape', function () {
        var a = tf.tensor1d([2]);
        var b = tf.tensor1d([4, 2, -1]);
        expect(function () { return tf.greaterEqualStrict(a, b); }).toThrowError();
        expect(function () { return tf.greaterEqualStrict(b, a); }).toThrowError();
    });
    // Tensor2D:
    it('Tensor2D - strict version throws when a and b are different shape', function () {
        var a = tf.tensor2d([[1.1], [7.1]], [2, 1], 'float32');
        var b = tf.tensor2d([[0.1, 1.1, 2.1], [7.1, 8.1, 9.1]], [2, 3], 'float32');
        expect(function () { return tf.greaterEqualStrict(a, b); }).toThrowError();
        expect(function () { return tf.greaterEqualStrict(b, a); }).toThrowError();
    });
    // Tensor3D:
    it('Tensor3D - strict version throws when a and b are different shape', function () {
        var a = tf.tensor3d([
            [[1.1, 0.1], [2.1, 3.1], [4.1, 5.1]],
            [[6.1, 7.1], [9.1, 8.1], [10.1, 11.1]]
        ], [2, 3, 2], 'float32');
        var b = tf.tensor3d([[[1.1], [2.1], [3.1]], [[7.1], [10.1], [9.1]]], [2, 3, 1], 'float32');
        expect(function () { return tf.greaterEqualStrict(a, b); }).toThrowError();
        expect(function () { return tf.greaterEqualStrict(b, a); }).toThrowError();
    });
    // Tensor4D:
    it('Tensor4D - strict version throws when a and b are different shape', function () {
        var a = tf.tensor4d([1.1, 2.1, 5.1, 9.1], [2, 2, 1, 1], 'float32');
        var b = tf.tensor4d([[[[1.1, 2.1]], [[3.1, 4.1]]], [[[5.1, 6.1]], [[7.1, 8.1]]]], [2, 2, 1, 2], 'float32');
        expect(function () { return tf.greaterEqualStrict(a, b); }).toThrowError();
        expect(function () { return tf.greaterEqualStrict(b, a); }).toThrowError();
    });
    it('accepts a tensor-like object', function () {
        var a = [1, 4, 5];
        var b = [2, 3, 5];
        var res = tf.greaterEqualStrict(a, b);
        expect(res.dtype).toBe('bool');
        test_util_1.expectArraysClose(res, [0, 1, 1]);
    });
});
//# sourceMappingURL=compare_ops_test.js.map