"use strict";
/**
 * @license
 * Copyright 2018 Google Inc. All Rights Reserved.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * =============================================================================
 */
Object.defineProperty(exports, "__esModule", { value: true });
var tf = require("../index");
var jasmine_util_1 = require("../jasmine_util");
var test_util_1 = require("../test_util");
jasmine_util_1.describeWithFlags('AdagradOptimizer', test_util_1.ALL_ENVS, function () {
    it('basic', function () {
        var learningRate = .1;
        var initialAccumulatorValue = .1;
        var optimizer = tf.train.adagrad(learningRate, initialAccumulatorValue);
        var x = tf.tensor1d([1, 2]).variable();
        var f = function () { return x.square().sum(); };
        var numTensors = tf.memory().numTensors;
        var cost = optimizer.minimize(f, /* returnCost */ true);
        // Cost & accumulator should be the only additional arrays.
        expect(tf.memory().numTensors).toBe(numTensors + 2);
        // epsilon = 1-e8
        // newAccumulatedGrad = accumulatedGrad + grad^2
        // x -= (learningRate * grad) / sqrt(newAccumulatedGrad + eps)
        // de/dx = [2, 4]
        // accumulatedGrad = [0.1, 0.1]
        // newAccumulatedGrad = [4.1, 16.1]
        // x = [0.9012270405, 1.900311042]
        test_util_1.expectArraysClose(x, [0.9012270405, 1.9003110428]);
        cost.dispose();
        numTensors = tf.memory().numTensors;
        cost = optimizer.minimize(f, /* returnCost */ false);
        // de/dx = [1.802454081, 3.9501555214]
        // accumulatedGrad = [4.1, 16.1]
        // newAccumulatedGrad = [7.3488407141, 31.7037286432]
        // x = [0.8347372764, 1.83015597828]
        // TODO: Fix numerical precision.
        test_util_1.expectArraysClose(x, [0.8347372764, 1.83015597828], 1e-2);
        // There should be no new additional Tensors.
        expect(tf.memory().numTensors).toBe(numTensors);
        expect(cost).toBe(null);
        x.dispose();
        optimizer.dispose();
        // The only tensor remaining is the argument to variable().
        expect(tf.memory().numTensors).toBe(1);
    });
    it('serialization round-trip', function () {
        var originalOpt = tf.train.adagrad(0.1, 0.2);
        var reserialized = tf.AdagradOptimizer.fromConfig(tf.AdagradOptimizer, originalOpt.getConfig());
        expect(reserialized.getConfig()).toEqual(originalOpt.getConfig());
    });
});
//# sourceMappingURL=adagrad_optimizer_test.js.map