"use strict";
/**
 * @license
 * Copyright 2018 Google Inc. All Rights Reserved.
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * =============================================================================
 */
Object.defineProperty(exports, "__esModule", { value: true });
var tf = require("../index");
var jasmine_util_1 = require("../jasmine_util");
var test_util_1 = require("../test_util");
jasmine_util_1.describeWithFlags('SGDOptimizer', test_util_1.ALL_ENVS, function () {
    it('basic', function () {
        var learningRate = .1;
        var optimizer = tf.train.sgd(learningRate);
        var x = tf.scalar(4).variable();
        var numTensors = tf.memory().numTensors;
        var cost = optimizer.minimize(function () { return x.square(); }, /* returnCost */ true);
        // Cost should be the only additional array.
        expect(tf.memory().numTensors).toBe(numTensors + 1);
        // de/dx = 2x
        var expectedValue1 = -2 * 4 * learningRate + 4;
        test_util_1.expectArraysClose(x, [expectedValue1]);
        test_util_1.expectArraysClose(cost, [Math.pow(4, 2)]);
        cost.dispose();
        numTensors = tf.memory().numTensors;
        cost = optimizer.minimize(function () { return x.square(); }, /* returnCost */ false);
        // There should be no new additional Tensors.
        expect(tf.memory().numTensors).toBe(numTensors);
        var expectedValue2 = -2 * expectedValue1 * learningRate + expectedValue1;
        test_util_1.expectArraysClose(x, [expectedValue2]);
        expect(cost).toBe(null);
        optimizer.dispose();
        x.dispose();
        // The only tensor remaining is the argument to variable().
        expect(tf.memory().numTensors).toBe(1);
    });
    it('serialization round-trip', function () {
        var learningRate = .1;
        var originalOpt = tf.train.sgd(learningRate);
        var reserialized = tf.SGDOptimizer.fromConfig(tf.SGDOptimizer, originalOpt.getConfig());
        expect(reserialized.getConfig()).toEqual(originalOpt.getConfig());
    });
});
//# sourceMappingURL=sgd_optimizer_test.js.map