"use strict";
var __values = (this && this.__values) || function (o) {
    var m = typeof Symbol === "function" && o[Symbol.iterator], i = 0;
    if (m) return m.call(o);
    return {
        next: function () {
            if (o && i >= o.length) o = void 0;
            return { value: o && o[i++], done: !o };
        }
    };
};
Object.defineProperty(exports, "__esModule", { value: true });
var tf = require("@tensorflow/tfjs");
var TimedMenu = (function () {
    function TimedMenu(config, callback, runtimeOptions) {
        this.config = config;
        this.callback = callback;
        if (runtimeOptions == null) {
            runtimeOptions = {};
        }
        this.stateSequence = [];
        this.tickMillis =
            runtimeOptions.tickMillis == null ? 500 : runtimeOptions.tickMillis;
        this.timeToLiveMultiplier =
            runtimeOptions.timeToLiveMultiplier == null ? 1 :
                runtimeOptions.timeToLiveMultiplier;
        this.intervalTask = setInterval(this.tick.bind(this), this.tickMillis);
        if (this.callback != null) {
            this.callback(this.stateSequence.map(function (node) { return node.name; }), 'init');
        }
    }
    TimedMenu.prototype.registerEvent = function (name) {
        var _this = this;
        var e_1, _a;
        var candidates = this.stateSequence.length === 0 ?
            this.config.nodes :
            this.stateSequence[this.stateSequence.length - 1].children;
        if (candidates == null) {
            throw new Error('Cannot handle more events');
        }
        var _loop_1 = function (candidate) {
            if (candidate.name === name) {
                if (candidate.children == null || candidate.children.length === 0) {
                    this_1.stateSequence.push(candidate);
                    if (this_1.callback != null) {
                        this_1.callback(this_1.stateSequence.map(function (node) { return node.name; }), 'leaf');
                    }
                    setTimeout(function () {
                        if (candidate.regressSteps != null && candidate.regressSteps > 0) {
                            for (var i = 0; i < candidate.regressSteps; ++i) {
                                _this.stateSequence.pop();
                            }
                            if (_this.callback != null) {
                                _this.callback(_this.stateSequence.map(function (node) { return node.name; }), 'regressSteps');
                            }
                        }
                        else {
                            _this.stateSequence = [];
                            if (_this.callback != null) {
                                _this.callback(_this.stateSequence.map(function (node) { return node.name; }), 'reset');
                            }
                        }
                    }, this_1.tickMillis);
                }
                else {
                    this_1.stateSequence.push(candidate);
                    if (this_1.callback != null) {
                        this_1.callback(this_1.stateSequence.map(function (node) { return node.name; }), 'advance');
                    }
                }
                this_1.lastEventTimeMillis = tf.util.now();
                return { value: candidate.action };
            }
        };
        var this_1 = this;
        try {
            for (var candidates_1 = __values(candidates), candidates_1_1 = candidates_1.next(); !candidates_1_1.done; candidates_1_1 = candidates_1.next()) {
                var candidate = candidates_1_1.value;
                var state_1 = _loop_1(candidate);
                if (typeof state_1 === "object")
                    return state_1.value;
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (candidates_1_1 && !candidates_1_1.done && (_a = candidates_1.return)) _a.call(candidates_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        return null;
    };
    TimedMenu.prototype.tick = function () {
        if (this.stateSequence != null && this.stateSequence.length > 0) {
            var currNode = this.stateSequence[this.stateSequence.length - 1];
            var nowMillis = tf.util.now();
            if (currNode.timeToLiveMillis > 0 &&
                nowMillis - this.lastEventTimeMillis >
                    currNode.timeToLiveMillis * this.timeToLiveMultiplier) {
                this.stateSequence.pop();
                this.lastEventTimeMillis = nowMillis;
                if (this.callback != null) {
                    this.callback(this.stateSequence.map(function (node) { return node.name; }), 'regress', currNode.timeOutAction);
                }
            }
        }
    };
    TimedMenu.prototype.stopTimer = function () {
        clearInterval(this.intervalTask);
    };
    return TimedMenu;
}());
exports.TimedMenu = TimedMenu;
//# sourceMappingURL=timed_menu.js.map